/*
 *   WDX: WitchDoctor-WDX, versatile distributed lock manager
 *   Copyright (C) 2003 Dominique Chabord,
 *   Please refer to credits section below
 *
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   ************************************************
 *
 *   History and credits:
 *   First version of WitchDoctor-WDX was based on:
 *
 *   DECwatchdog/Autopilot, watchdog and lock manager
 *   Copyright (C) 2003 HP
 *
 *   Developed by
 *     Computer Special Systems, Annecy France
 *     A Division of Digital Equipment Corporation
 *   and
 *     Custom-Systems, Annecy France
 *     An engineering center of Compaq
 *
 *   Provided under GNU-GPL terms by HP, 30 June 2003
 *   GNU-GPL replaces previous license statements
 *
 *   Rebranded, ported and modified for Shaman-X project, 15 July 2003
 *   by Dominique Chabord
 *   and maintained by other contributors as mentionned in the code
 *
 *   ************************************************
 *
 *   WDX is promoted by Shaman-X project
 *
 *   Check for new versions and documentation at
 *   http://www.shaman-x.org
 *
 *   ************************************************
 *
 *   Most of WDX code is designed to run unchanged on
 *   different operating systems, including at V1.0 time:
 *   Gnu-Linux, WindowsNT, DigitalUNIX and OpenVMS.
 *   Operating system specifics are identified by dedicated files.
 *
 *   Evolutions
 *   ----------
 *   July-2003     Dominique Chabord    WitchDoctor-WDX V1.0
 *                 Creation
 *
 *   ************************************************
 *
 *
 *	wdx.h
 *
 *	Global definitions, data structures, and constants
 *
 ************************************************************************/
/*
 *
 *  17-Feb-94 -- Cornu
 *      Creation
 * MK001 - M. Kennealy   Added new parameter to wat_local_member_transition
 * MK002 - M. Kennealy   Changed the struct interface to interface_char
 *                       in order to prevent a clash with a special
 *                       keyword in Windows NT.
 */
#ifndef		WDX_H
#define		WDX_H

#include	"wdx_specific.h"
#include	"wdx_api.h"


/*
 * Parameter limits : (They should NOT be changed at all)
 */

#define WDX_VERSION     004    /* version of software : 0.4 */
#define WDX_VERSION_STRING "V0.4"

/* format of the version field in config file */
#define VERSION_WORDS_COUNT 2 /* format of the version field x.y */
#define DATE_LEN   20       /* MAX strlen of date string */

#define MISSION_LEN     17     /* MAX strlen for MISSION name */

#define	RESOURCE_COUNT  (1+24) /* MAX number of resources */
#define	RESOURCE_LEN    21     /* MAX strlen of resource names */

#define SNAPSHOT_LEN    9      /* MAX length for snapshot */

#define MASTER_INDEX    0      /* resource[0] is reserved for MASTERTOKEN */
#define FIRST_RESOURCE  1      /* user resource index begins at 1 */
#define NOT_AN_INDEX    -1     /* not an index */
#define NOT_AN_OWNER    -1     /* not an owner */
#define NOT_A_PID       -1     /* not a pid */

#define	MASTER_TOKEN        "masterbybcdefghijklm" /* resource which identifies the master */
#define MASTER_TOKEN_STRING "MASTERTOKEN"
#define MASTERHOLDER_IS_UNKNOWN   "masterholder.is.unknown"

#define MEMBER_COUNT    16     /* MAX number of members */
#define	MEMBER_LEN      32     /* MAX strlen of member name 	*/

#define STRING_LEN      256    /* MAX length for local strings */
#define SCRIPT_LEN      (STRING_LEN/2) /* sizeof script string */

#define DEFAULT_MISSION "wdx"  /* default mission name */

#define MAX_PROCESS_ON_APPLICATION 3  /* API max constant */
#define MAX_APPLICATION_ON_PROCESS 30 /* API max constant */
#define MAX_SOCKET	(MAX_PROCESS_ON_APPLICATION+1)
#define MAX_FIFO	(MAX_APPLICATION_ON_PROCESS+1)

#define FIFO_LEN	128    /* MAX strlen of a FIFO */

#define PAD_LEN         3      /* number of bytes in pad */

#define	COMMAND_LEN    FIFO_LEN /* MAX strlen of an command argument */
#define	OPERATION_LEN  FIFO_LEN /* MAX strlen of an operation argument */

/*
 * File name definition
 */
#define DEFAULT_PROG_NAME	"wdx"		/* default name for the binary */

#define DATA_BASE	"%s%s.dat"	/* data base generic name */
#define DATA_EXTENSION  ".dat"

#define LOG_FILE        "%s%s/%s.log"	/* log file generic name */

#define MISSION_FILE	"%s%s/%s.cfg"	/* mission file name definition */
#define MISSION_EXTENSION ".cfg"	/* extension for the mission file */

#define SUPPORT_FILE    "%s%s/%s.sup"	/* support file name definition */
#define SUPPORT_EXTENSION ".sup"        /* extension for the support file */

#define MEMBER_FILE    "%s%s/%s.mip"	/* members_ip file name definition */

#define LOCAL_FILE      "%s%s/%s.loc"	/* local file name definition */
#define LOCAL_EXTENSION ".loc"		/* extension of the local file */

#define SNAPSHOT_FILE   "%s%s/%s.hot"	/* snapshot file name definition */
#define SNAPSHOT_EXTENSION ".hot"

#define SNAPASCII_FILE  "%s%s/%s.txt"	/* ASCII snapshot file name */
#define SNAPASCII_EXTENSION ".txt"	/* extension of the ASCII snapshot */


/*
 * Activity algorithm definition
 */
#define ACTIVITY_CONSTANT 1   /* used by activity algorithm */
#define ACTIVITY_TRIGGERING 1 /* used by activity algorithm */

/*
 * Data number definitions
 */
#define DATA_NUMBER_OPERATION 0   /* datanumber field for operation messages */
#define DATA_NUMBER_INITIALIZED 0 /* init value for datanumber tables */
/* idle and running message from 10000 to 2^32 -1 */
#define DATA_NUMBER_RUNNING_OFFSET 10000
/* silence message from 10 to 10000 */
#define DATA_NUMBER_SILENCE_OFFSET 10

/*
 * Internal timer definition
 */
#define PERIOD_VAL 1		/* default scheduler time base */

#define END_VALIDITY    -1      /* timeout for timer event raising */

/* possible values for inter_flag field */
enum timer_flag { FLAGRESET, FLAGSET };

/* possible values for inter_event field */
enum timer_event { EVENTRESET, EVENTSET};

/* defines interface timer structure */
struct wdx_timer {
enum timer_flag   flag;
enum timer_event event;
int		 value;
};

#define THOUSAND 1000
#define MILLION 1000000
/* macro to convert nanoseconds to microseconds */
#define nano_to_micro(a) ((a)/THOUSAND)
/* macro to convert milliseconds to microseconds */
#define milli_to_micro(a) ((a)*THOUSAND)

/*
 * New type definition
 */
typedef char WDX_BYTE;
typedef struct _WDX_INT
{
unsigned char c0;
unsigned char c1;
unsigned char c2;
unsigned char c3;
} WDX_INT;


typedef struct _WDX_SHORT
{
unsigned char c0;
unsigned char c1;
} WDX_SHORT;


/*
 * Definition used for checksum calculation
 */
#define INT_SIZE (sizeof(unsigned int))
#define BYTE_IN_BITS 8				/* 8 bits */
#define INT_IN_BITS  (INT_SIZE * BYTE_IN_BITS ) /* 32 bits */


/*
 * LICENSE management definitions
 */

#define PROD_VERSION_MAJOR_NB 0
#define PROD_VERSION_MINOR_NB 4

#define PRODUCER "Shaman-X"

#define FLAGS   0
#define MASTER_SUBSTITUTE_ID 1



/*
 * PRINT format definition
 */
#define MISSION_FORMAT         (MISSION_LEN + 6)
#define MEMBER_FORMAT          ((MEMBER_LEN/2) + 6)
#define MEMBER_STATUS_FORMAT   14
#define NETWORK_FORMAT         MEMBER_STATUS_FORMAT
#define RESOURCE_FORMAT        MEMBER_FORMAT
#define RESOURCE_STATUS_FORMAT MEMBER_STATUS_FORMAT
#define OWNER_FORMAT           MEMBER_STATUS_FORMAT
#define GENERATION_FORMAT      MEMBER_STATUS_FORMAT

#define SNAPSHOT_CFG      "MISSION CONFIGURATION FILE"
#define SNAPSHOT_LOC      "LOCAL CONFIGURATION FILE"
#define SNAPSHOT_CONSTANT "CONSTANT INFORMATION"
#define SNAPSHOT_LIST     "LIST INFORMATION"
#define SNAPSHOT_INTERVAL "--------------------------------------------------------------------------------"
#define SNAPSHOT_TITLE    "SNAPSHOT                                            date : "

#define MEMBER_TRANSITION_FORMAT_MESSAGE "Member %s %s to %s"
#define MEMBER_TRANSITION_FORMAT_SCRIPT  "%s MEMBER %s %s %s"
#define NETWORK_TRANSITION_FORMAT_MESSAGE "For member %s, network %d (%s) %s to %s"
#define NETWORK_TRANSITION_FORMAT_SCRIPT "%s NETWORK %s %d %s %s %s"
#define RESOURCE_TRANSITION_FORMAT_MESSAGE "Resource %s %s by %s"
#define RESOURCE_TRANSITION_FORMAT_SCRIPT "%s RESOURCE %s %s %s %s"
#define MISMATCH_TRANSITION_FORMAT_MESSAGE "Mismatch for resource %s by %s"
#define MISMATCH_TRANSITION_FORMAT_SCRIPT "%s MISMATCH %s %s"

#define PRINTABLE_YES "YES"
#define PRINTABLE_NO "NO"
#define PRINTABLE_MAS_SUB "Masterholder - Substitute"
#define PRINTABLE_SAT  "Satellite"
/*
/* Message management macros definitions
 */

#define WDX_MESS_E 0
#define WDX_MESS_I 1
#define WDX_MESS_W 2
#define WDX_MESS_C 3
#define ERROR_MESSAGE "ERROR   : "
#define WARNING_MESSAGE "WARNING : "
#define INFOR_MESSAGE "INFO    : "
#define COMPLEMENT_MESSAGE ""

#define PERROR_B lib_output_message(WDX_MESS_E,
#define PERROR_E ,(char *)0)

#define PWARNING_B lib_output_message(WDX_MESS_W,
#define PWARNING_E ,(char *)0)

#define PINFOR_B lib_output_message (WDX_MESS_I,
#define PINFOR_E ,(char *)0)

#define PCOMPLEMENT_B  lib_output_message (WDX_MESS_C,
#define PCOMPLEMENT_E ,(char *)0)

#define ERRNO_TOKEN     "%m"
#define DATE_TOKEN      "%D"
#define MAX_ARG         100

/* macros to convert an enum to a string */
#define MAX_STATUS_LENGTH 20    /* max length os a string */
#define ENUM_TO_STRING(a1,a2) ((a1>=0&&a1<=(sizeof(a2)/MAX_STATUS_LENGTH)) ? &a2[a1][0] : "unknown")
#define resource_status_to_s(a) ENUM_TO_STRING(a,resourcestatustab)
#define member_status_to_s(a)   ENUM_TO_STRING(a,memberstatustab)
#define network_status_to_s(a)  ENUM_TO_STRING(a,networkstatustab)
#define lock_transition_to_s(a) ENUM_TO_STRING(a,locktransitiontab)
#define lock_request_to_s(a)    ENUM_TO_STRING(a,lockrequesttab)
#define lock_answer_to_s(a)     ENUM_TO_STRING(a,lockanswertab)
#define event_type_to_s(a)      ENUM_TO_STRING(a,eventtypetab)



/*
 * Internal coding of various constants
 * enum and associated character string tables
 */
enum mask { NO, YES };
enum flag { DISABLE, ENABLE };

/* application command types */
enum wdx_type { WDX_NO_TYPE,		/* RESERVED */
		WDX_COMMAND,
		WDX_REQUEST,
		WDX_LOCAL };

/* operational watchdog status */
enum    watchdog_status { W_FAULT,
			  W_IDLE,
			  W_RUNNING };

#define WATCH_SUBSTATUS 6  /* number of operational substatus */

enum    watchdog_substatus { SUBIDLE,	        /* real idle state */
			     SUBCOLD,           /* cold startup request */
			     SUBWARM,           /* warm startup request */
			     SUBSILENCE,        /* silence period at startup */
			     SUBRUNNING,        /* real running state */
			     SUBSTOPPING,       /* stop request */
			     SUBFAULT
			   };

/* operational status of a member in the group */
#define MEMBER_STATUS 5	/* number of values in member_status definition */
enum	member_status	{ FAULTY, IDLE,
			  MASTERHOLDER, SUBSTITUTE, SATELLITE };

/* string associated to memberstatus */
#ifdef MAIN
char memberstatustab[5][MAX_STATUS_LENGTH] ={
                        "fault",
                        "idle",
                        "masterholder",
                        "substitute",
                        "satellite"};

#else /* MAIN */
extern char memberstatustab[5][MAX_STATUS_LENGTH];
#endif /* MAIN */


/* operational member type */
enum	member_type	{ MASTER_SUBSTITUTE_T, SATELLITE_T };

/* operational process type */
enum    wdx_modes	{ WDX_PROCESS, WDX_APPLICATION };

/* startup mode */
enum init_mode { COLD_INIT_PROCESS, COLD_REINIT_PROCESS, WARM_REINIT_PROCESS };


/*
 * API control definitions
 *
 */

/* values not used by OPERATION messages but provided to the API */
#define WDX_OPEN_CONTROL  60
#define WDX_CLOSE_CONTROL 61

/* API open control mode */
enum control_mode { NOT_A_MODE,
		    REQUEST_CONTROL_ONLY,
	 	    EVENT_CONTROL_ONLY,
		    REQUEST_AND_EVENT_CONTROL };


/*
 * mask_element data structure
 */
struct mask_element
{
	enum mask value;	/* mask value */
        char      *key;		/* pt to string */
};



/*
 * COMMAND messages definitions
 */

/* possible value for command type */
enum command_type    { TNOTYPE,
		       TREQUEST,
		       TANSWER,
		       TACTION
                     };

/* possible value for command_request */
enum command_request { RNOREQUEST,
		       RRUN,
		       RSTOP,
		       RSTART,
                       RPAUSE,
                       RPAUSESTART
		     };

enum command_answer { ANOANSWER,
		      AOK,
		      ANONOK
		    };

enum command_action { CNOACTION,
		      ACRUN,
		      ACKILL,
		      ACSTOP,
		      ACSTART,
		      ACSHUT,
		      ACUP,
		      ACLOCKILL,
		      ACTIMER,
                      ACPAUSE,
                      ACPAUSETIMER,
                      ACPAUSESTART
		    };

enum startup_mode { COLD_INIT, COLD_REINIT, WARM_REINIT };

/*
 * c_request, c_answer and c_action data structure are used by the
 * command_protocol for command manipulation.
 * They are forwarded over the network into the 'command' data structure.
 * These three structures have a format designed to fill into the command
 * data structure.
 */
struct c_request
{
    enum command_request  cr;    /* command request value */
    int                   gn;    /* generation number */
    unsigned int	  ar;    /* argument of the request */
};

struct c_answer
{
    enum command_answer   ca; /* command answer value */
    int                   ri; /* requester index to select the right request */
    enum command_request  cr; /* command request value to select */
    int                   gn; /* generation number to select */
};

struct c_action
{
    enum command_action   cc; /* command action value */
    int			  a1; /* action argument : NOT USED */
    int			  a2; /* action argument : NOT USED */
    int			  a3; /* action argument : NOT USED */
    char     dt[COMMAND_LEN]; /* action data */
};


/*
 * Lock and wait request definitions
 */

/* possible value for wait request */
enum wait_operation 	{ WAIT_FOR_MOUNT,
			  WAIT_FOR_RESERVE
			};

/* possible value for lock request */
enum    lock_request    { NOREQUEST,
                          DECLARING,
                          ERASING,
                          RESERVING,
                          MOUNTING,
                          FREEING,
                          FAULTING,
                          GETTING
                        };
#ifdef MAIN
char lockrequesttab[8][MAX_STATUS_LENGTH] ={
                        "norequest",
                        "declaring",
                        "erasing",
                        "reserving",
                        "mounting",
                        "freeing",
                        "faulting",
                        "getting"
                        };
#else /* MAIN */
extern char lockrequesttab[8][MAX_STATUS_LENGTH];
#endif /* MAIN */


/* possible value for lock answer */
enum    lock_answer     { NOANSWER,
                          OK,
                          NONOK };
#ifdef MAIN
char lockanswertab[3][MAX_STATUS_LENGTH] ={
                        "noanswer",
                        "ok",
                        "nonok"};
#else /* MAIN */
extern char lockanswertab[3][MAX_STATUS_LENGTH];
#endif /* MAIN */


/* possible value for lock transition */
enum	lock_transition { NOTRANS,
                          DECLARE,
                          ERASE,
                          RESERVE,
                          MOUNT,
                          FREET,
                          FAULTT,
                          GETT
                        };
#ifdef MAIN
char locktransitiontab[8][MAX_STATUS_LENGTH] ={
                        "notrans",
                        "declare",
                        "erase",
                        "reserve",
                        "mount",
                        "free",
                        "fault",
                        "get"};

#else /* MAIN */
extern char locktransitiontab[8][MAX_STATUS_LENGTH];
#endif /* MAIN */


/*
 * The structure resourcedef is used to store the resource characteristics
 */

/* possible locking status of a local resource	*/
#define RESOURCE_STATUS 5
enum	resource_status   { NOTKNOWN, FREE, FAULT, RESERVED, MOUNTED };


#ifdef MAIN
char resourcestatustab[5][MAX_STATUS_LENGTH] ={
                        "unknown",
                        "free",
                        "fault",
                        "reserved",
                        "mounted"};
#else /* MAIN */
extern char resourcestatustab[5][MAX_STATUS_LENGTH];
#endif /* MAIN */

/* possible resource event type */
enum    event_type      { NOEVENT,
                          REQUEST,
		          ANSWER,
                          TRANSITION,
                          INFORMATION,
                          SHORTCUT };

#ifdef MAIN
char eventtypetab[5][MAX_STATUS_LENGTH] ={
                        "request",
                        "answer",
                        "transition",
                        "information",
	                "shortcut" };

#else /* MAIN */
extern char eventtypetab[4][MAX_STATUS_LENGTH];
#endif /* MAIN */

enum lock_return { LOCK_R,  /* rejected */
		   LOCK_T,  /* too much resources */
		   LOCK_S,  /* success */
		   LOCK_M,  /* not a local mount point */
		   LOCK_D,  /* lock already done */
		   LOCK_P   /* get lock not allowed : it's a mount point */
		 };

/* possible valued for mark */
enum resource_marked { NOT_MARKED, MARKED };

/* possible values for clean */
enum resource_cleaned { NOT_CLEANED, CLEANED };

/* When the resource is locked ( status RESERVED, MOUNTED and FAULT ) the
 * initiator is also called the owner
 */
struct resource_def
{
    char             name[RESOURCE_LEN]; /* name of the resource */
    enum resource_status         status; /* status of the resource */
    enum resource_status      oldstatus; /* old status of the resource */
    time_t                      generation; /* generation date of the resource */
    int                       initiator; /* index of the request initiator */
    int                         oldinit; /* old initiator */
    enum resource_marked           flag; /* MARKED or NOT_MARKED */
    enum resource_cleaned         clean; /* CLEANED or NOT_CLEANED */
    struct wdx_timer              delay; /* delay timer for script activation */
};


/*
 * WDX communication data structures :
 *      MESSAGE,
 *      HEARTBEAT,
 *      WATCHDOG,
 *      RESOURCE, COMMAND and OPERATION
 *      PROCESS_MESSAGE
 */

/* possible structures sent via UDP		*/
enum	msg_type	{ HEARTBEAT, COMMAND, OPERATION };

/*
 * The struct header is the HEAD of the struct message
 */
struct header
{
    unsigned char     wdxversion; /* wdx version compiled-in value */
    unsigned int   configversion; /* value returned by config_checksum() */
    int               missionkey; /* key calculated on mission name */
    int              memberindex; /* index of the member */
    pid_t              memberpid; /* member pid */
    unsigned int      datanumber; /* message number used to filter messages */
    enum msg_type       datatype; /* message type */
    int                 datasize; /* size of data */
};

struct encoded_header
{
    WDX_BYTE          wdxversion; /* wdx version compiled-in value */
    WDX_INT        configversion; /* value returned by config_checksum() */
    WDX_INT           missionkey; /* key calculated on mission name */
    WDX_INT          memberindex; /* index of the member */
    WDX_INT            memberpid; /* member pid */
    WDX_INT           datanumber; /* message number used to filter messages */
    WDX_INT             datatype; /* message type */
    WDX_INT             datasize; /* size of data */
};

/*
 * The struct watchdog is used by the watchdog and failover protocol
 */
struct watchdog
{
    int                 index; /* index of the member */
    enum member_status status; /* status of the member */
    int              validity; /* validity time for the message */
};

struct encoded_watchdog
{
    WDX_INT            index; /* index of the member */
    WDX_INT           status; /* status of the member */
    WDX_INT         validity; /* validity time for the message */
};

/*
 * The struct resource include the data broadcasted by the locking protocol
 */
struct resource
{
    char             name[RESOURCE_LEN]; /* name of the resource */
    enum event_type             evttype; /* type of event */
    int                          value1; /* data value 1 */
    int                          value2; /* data value 2 */
    int                          value3; /* data value 3 */
};

struct encoded_resource
{
    WDX_BYTE         name[RESOURCE_LEN]; /* name of the resource */
    WDX_INT                     evttype; /* type of event */
    WDX_INT                      value1; /* data value 1 */
    WDX_INT                      value2; /* data value 2 */
    WDX_INT                      value3; /* data value 3 */
};

/*
 * The struct heartbeat is the HEARTBEAT message definition
 */
struct heartbeat
{
   struct watchdog wat;
   struct resource res[RESOURCE_COUNT];
};

struct encoded_heartbeat
{
   struct encoded_watchdog wat;
   struct encoded_resource res[RESOURCE_COUNT];
};


/*
 * The struct command is the COMMAND message definition
 */
struct command
{
    enum command_type        type; /* command type */
    int                     index; /* index of the sender member */
    int		           value1; /* command argument 1 */
    int		           value2; /* command argument 2 */
    int		           value3; /* command argument 3 */
    int		           value4; /* command argument 4 */
    char commanddata[COMMAND_LEN]; /* data of the operation */
};

struct encoded_command
{
    WDX_INT                     index; /* index of the sender */
    WDX_INT                      type; /* command type */
    WDX_INT	               value1; /* command argument 1 */
    WDX_INT	               value2; /* command argument 2 */
    WDX_INT	               value3; /* command argument 3 */
    WDX_INT    	               value4; /* command argument 4 */
    WDX_BYTE commanddata[COMMAND_LEN]; /* data of the operation */
};

/*
 * The struct operation is the OPERATION message definition
 */
struct operation
{
    int                       index; /* index of the sender member */
    pid_t                       pid; /* pid of the sender process */
    int                        type; /* operation type */
    int		         identifier; /* request identifier */
    char operationdata[OPERATION_LEN]; /* data of the operation */
    int 	  operationargument; /* operation argument */
};

struct encoded_operation
{
    WDX_INT                         index; /* index of the sender */
    WDX_INT                           pid; /* pid of the sender process */
    WDX_INT                          type; /* operation type */
    WDX_INT	  	           identifier; /* request identifier */
    WDX_BYTE operationdata[OPERATION_LEN]; /* data of the operation */
    WDX_INT 	    operationargument; /* operation argument */
};

/*
 * The struct message is the GENERIC structure used for communication
 * The sizeof(struct message) must not exceed 1024 bytes, which is
 * the size of a single UDP/IP message.
 */
struct message
{
    struct header head;
    union
    {
    struct heartbeat h;
    struct command   c;
    struct operation o;
    } m;
    unsigned char pad[PAD_LEN];
};


struct encoded_message
{
    WDX_INT           checksum;     /* checksum on encoded message */
    struct encoded_header head;
    union
    {
    struct encoded_heartbeat h;
    struct encoded_command c;
    struct encoded_operation o;
    } m;
    unsigned char pad[PAD_LEN];
};

#define CHECK_MESSAGE head
#define SIZE_MESSAGE ((sizeof(struct encoded_message)-sizeof(unsigned int)))
#define WDX_SOCKET_DATA_SIZE (sizeof(struct encoded_message))


/*
 * Process_message structure : used by process to send message to application
 */

enum process_type { PROCESS_ACKNOWLEDGE,
		    PROCESS_NOTIFICATION,
		    PROCESS_EVENT,
		    PROCESS_CONTROL };

enum process_status { PROCESS_ERROR,
		      PROCESS_WARNING,
		      PROCESS_SUCCESS,
		      PROCESS_FIRST,
		      PROCESS_LAST,
		      PROCESS_TRASHED };

#define PROCESS_DATA_LEN (4096-(5*sizeof(int)))
#define SYN_NUMBER_OF_MESSAGES 1

struct process_acknowledge
{
    enum process_type      type; /* message type */
    enum process_status  status; /* status */
    int              identifier; /* command id : optional */
    int              generation; /* generation number */
    char data[PROCESS_DATA_LEN]; /* data length */
};

#define PROCESS_MESSAGE_LEN (128-(6*sizeof(int)))
#define ASYN_NUMBER_OF_MESSAGES 32

struct process_message
{
    enum process_type       type; /* message type */
    enum process_status   status; /* status */
    int               identifier; /* command id : optional */
    int               generation; /* generation number */
    enum resource_status rstatus; /* resource status notification */
    char data[PROCESS_MESSAGE_LEN]; /* data length */
};


/*
 * COMMUNICATION INTERFACES definitions
 */

#define NETWORKS 2
#define NETWORK_OFFSET 1

/* possible values for interface use field */
enum    inter_use { NOTUSED, USED };

/* possible values for inter_transition field */
enum inter_transition { NETSTATUSRESET,NETSTATUSSET };

/* possible values for interface stat field */
#define NETWORK_STATUS 4
enum    inter_status  { NA, UNKNOWN, ON, OFF };
#ifdef MAIN /* refers to intyerstatus enum */
char networkstatustab[4][MAX_STATUS_LENGTH] ={
                        "n/a",
                        "unknown",
                        "on",
                        "off"};

#else /* MAIN */
extern char networkstatustab[4][MAX_STATUS_LENGTH];
#endif /* MAIN */


/* networks */
enum networks { NETWORK_1, NETWORK_2 };

/* defines interface characteristics */
struct interface_char {          /* MK002 */
   char        ifname[IFNAMSIZ]; /* name of the interface */
   enum networks          netnb; /* network number */
   enum inter_use           use; /* NA or USED */
   struct wdx_timer    validity; /* delta time value used for validity */
   struct wdx_timer     confirm; /* confirmation delay of networks */
   enum inter_status     status; /* network status */
   enum inter_status    mstatus; /* network memorized status */
   enum inter_transition   flag; /* network transition flag */
   enum flag          threshold; /* threshold feature */
   int                    value; /* threshold value */
   int			reached; /* reached flag 0 non reached / 1 reached */
   int                    count; /* number of threshold reached */
   int                      max; /* max interval between two messages */
};

/* interfaces_used structure is used to call open_socket() subroutine */
struct interfaces_used
{
   char ifname[IFNAMSIZ];     /* name of the interface */
   enum inter_use    use;     /* NA or USED */
};


/*
 * IO data structure : socket and fifo input/output structures
 */


/* output structure is used to call send_socket_message() subroutine */
struct socket_output
{
    int		 descriptor;		/* desc. for communication module */
    char            *buffer;            /* buffer pointer */
    int                size;            /* buffer size */
};

/* input structure is used to call get_socket_message() subroutine */
struct socket_input
{
    int	         descriptor;	       /* desc. for communication module */
    char            *buffer;           /* buffer pointer */
    int                size;           /* buffer size */
    int              number;           /* network on which mess. was received */
};

/* output structure is used to call send_fifo_message() subroutine */
struct fifo_output
{
    int		 descriptor;		/* desc. for communication module */
    char            *buffer;            /* buffer pointer */
    int                size;            /* buffer size */
};

/* input structure is used to call get_socket_message() subroutine */
struct fifo_input
{
    int	         descriptor;	       /* desc. for communication module */
    char            *buffer;           /* buffer pointer */
    int                size;           /* buffer size */
};


/*
 * GROUP, MEMBER and MEMBER_BASE definitions
 */

/* The struct member defines characteristics of a machine being part of
 * the group
 */

struct member
{
    char          name[MEMBER_LEN];	/* internet name of machine	*/
    enum member_type          type;	/* Is this host master candidate ? */
    enum member_status      status;     /* member status */
    enum member_status   brdstatus;     /* received status value */
    int			  priority;	/* mastertoken priority affectation */
    int                  broadcast;     /* config broadcast value */
    int                   validity;     /* config validity value */
    int                 redundancy;     /* redundancy value */
    struct interface_char networks[NETWORKS]; /* interface status MK002 */
};


/* The struct group describes the common view of the group
 * It includes the lifesigns of all members,
 * as well as the static configuration data.
 * The index of a member is the member's offset in the memberlist.
 *
 * This structure can be shared throughout the group.
 *
 */
struct group
{
int                                localindex; /* local index in table */
char                               wdxversion; /* wdx version */
unsigned int                    configversion; /* config file version */
unsigned char    version[VERSION_WORDS_COUNT]; /* version of configuration */
char                         date[DATE_LEN+1]; /* date of configuration */
int				   missionkey; /* mission key */
int                               membercount; /* member count */
int                             resourcecount; /* count of resources */
enum flag		                  log; /* log feature flag */
char                      logfile[SCRIPT_LEN]; /* log file name */
int                                  priority; /* process priority value */
int                                    period; /* scheduler period */
enum flag                            activity; /* process activity flag */
enum flag                          statistics; /* statistics flag */
int                          statisticsperiod; /* statistics period */
char                     mission[MISSION_LEN]; /* mission name */
char           nosubstitutescript[SCRIPT_LEN]; /* nosubstitutescript def */
char              newmasterscript[SCRIPT_LEN]; /* newmasterscript def */
char              satellitescript[SCRIPT_LEN]; /* satellitescript def */
char              exceptionscript[SCRIPT_LEN]; /* exceptionscript def */
char                  eventscript[SCRIPT_LEN]; /* eventscript */
/* networkmask table */
struct mask_element networkmasks[NETWORK_STATUS][NETWORK_STATUS];
/* membermask table */
struct mask_element membermasks[MEMBER_STATUS][MEMBER_STATUS];
/* resourcemask table */
struct mask_element resourcemasks[RESOURCE_STATUS][RESOURCE_STATUS];
/* mismatchmask value */
struct mask_element mismatchmask;
struct member        memberlist[MEMBER_COUNT];    /* member data base */
struct resource_def resourcelist[RESOURCE_COUNT]; /* resource data base */
};


struct  member_base
{
  char          name[MEMBER_LEN]; /* internet name of machine	*/
  enum member_type          type; /* member type */
  pid_t			     pid; /* pid of the wdx process */
  int		      localindex; /* index of the process in group table */
  int                 redundancy; /* redundancy value */
  char          udp[MISSION_LEN]; /* udp port name */
  struct interfaces_used links[NETWORKS]; /* interfaces definition */
  unsigned int          checksum; /* config checksum */
  int	                     key; /* mission key */
  char	                 version; /* software version */
};



/*
 * Eventscript data structure used to call the lib_eventscript() routine
 */

#define MASTER_DISAPPEARENCE "MASTERDISAPPEARENCE"
#define MASTER_CHANGE        "MASTERCHANGE"

#define OPERATOR        "OPERATOR"
#define DOUBLE_MASTER   "DOUBLEMASTER"

enum eventscript_type { EVT_RESOURCE,
		        EVT_MISMATCH,
			EVT_MEMBER,
			EVT_NETWORK };

struct member_evt {
enum member_status old; /* old status */
enum member_status new; /* new status */
};

struct network_evt {
enum networks	   net; /* network number */
enum inter_status  old; /* old status */
enum inter_status  new; /* new status */
};

struct resource_evt {
char name[RESOURCE_LEN+1]; /* resource name */
enum resource_status old; /* old status */
enum resource_status new; /* new status */
};

struct mismatch_evt {
char name[RESOURCE_LEN+1]; /* resource name */
};

struct eventscript_data
{
enum eventscript_type  type; /* type of event */
enum mask	     script; /* script activity */
enum mask           display; /* display activity */
struct group         *group; /* pt to group data base */
int                   index; /* index of the member concerned */
union {
    struct member_evt     m;
    struct network_evt    n;
    struct resource_evt   r;
    struct mismatch_evt   i;
      } data;
};

/*
 * Mission list queue : returnd by spc_get_running_mission()
 */
struct mission_list
{
  char mission[MISSION_LEN];
  struct mission_list *next;
};


#ifdef MAIN
#define	EXTERN
#else
#define	EXTERN extern
#endif /* MAIN */

EXTERN	char	*pn;			  /* the program's name */
/* */
EXTERN	char	mastertoken[RESOURCE_LEN]; /* name for master token */
/* */
EXTERN  char        localname[MEMBER_LEN]; /* local member name */
EXTERN	int	           localbroadcast; /* period for messages */
EXTERN  int                    localindex; /* index of the local member */
EXTERN  unsigned int           oldversion; /* old config version */
/* */
EXTERN  char    messagestring[STRING_LEN]; /* message buffer */
/* */
EXTERN  int              socketdescriptor; /* socket communication descriptor */
/* */
EXTERN  struct group                  ggs; /* global group structure */
EXTERN  struct message            message; /* message data structure */
/* */
EXTERN  char                *wdxdirectory; /* watchdog directory name */

EXTERN	struct in_addr members_ip[MEMBER_COUNT];	/* members IP addresses */
EXTERN	int	using_ip_addresses;			/* non zero if using IP addresses */


/*
 * PUBLIC ROUTINES PROTOTYPES
 */

/* library routines */
//void lib_char_to_i(WDX_BYTE *ch,WDX_INT  *i);
void lib_char_to_i(WDX_BYTE *ch, unsigned int *i);
//void lib_i_to_char(WDX_INT  *i,WDX_BYTE *ch);
void lib_i_to_char(unsigned int *i, WDX_BYTE *ch);
void lib_encode_message(struct message *mp,struct encoded_message *emp);
void lib_decode_message(struct encoded_message *emp,struct message *mp);
void lib_forward_operation(int des,int t,int i,char *d,int a);
void lib_forward_command(int des,enum command_type t,char *d,unsigned int n);
void lib_forward_announce(struct message *mess,struct member *localmemberp,unsigned int dtn);
unsigned int lib_checksum(unsigned char *l,int size);
int lib_redirect_output(enum flag flg,char *log);
void lib_output_message(int first,...);
char *lib_generation_to_s(time_t *g);
int lib_index_of_resource(struct resource_def *rsd,char *res);
enum resource_status lib_status_of_resource(struct resource_def *rsd,char *p);
int lib_requester_of_resource(struct resource_def *rsd,char *p);
int lib_is_member_valid(struct member *mfp,int ind);
int lib_create_process_info(char *m);
int lib_delete_process_info(char *m);
int lib_set_process_info(char *m,struct member_base *p);
int lib_get_process_info(char *m,struct member_base *p);
int lib_set_snapshot(char *s,struct group *g);
int lib_get_snapshot(char *s,char *m,struct group *g);
int lib_process_init(enum init_mode c,char *s,struct group *g1,struct group *g2, struct message *m);
time_t lib_build_generation();
void lib_timer_reset(struct wdx_timer *p);
void lib_timer_set(struct wdx_timer *p, int t);
void lib_timer_decrease(struct wdx_timer *p,int t);
int  lib_timer_valid(struct wdx_timer *p);
int  lib_timer_alive(struct wdx_timer *p);
int  lib_timer_event(struct wdx_timer *p);
int  lib_timer_reached(struct wdx_timer *p,int t);
void lib_eventscript(struct eventscript_data *p);
int  lib_is_numeric(char *s);

/* lockprotocol routines */
void lck_init(enum startup_mode s,struct group *g,struct message *m);
int  lck_ins_information_in_newlist(int *flgpt);
void lck_delete(void);
int  lck_validate(int decrement,int *brdflgpt);
void lck_purge_requester(int ow,enum member_status status,int timeout);
void lck_purge_resource_delayed();
void lck_reset_resource();
void lck_activity_set(enum mask lockvalue, enum mask scriptvalue);
void lck_unmark_resource();
int  lck_resource_input(int id,struct resource rt[],int *brdflgpt);
int  lck_lock_operation_input(struct operation *c,enum lock_request t, int *brdflgpt);
int  lck_wait_operation_input(struct operation *c, enum wait_operation t);
int  lck_automatic_request_wish(char *res,int *brdflgpt);

/* config routines */
int cfg_config_init(char *missionpt,struct group *g);

/* communication routines */
int com_open_socket(char *s,int r,struct interfaces_used *i,enum wdx_modes h);
int com_update_socket(int d,int r,struct interfaces_used  *i);
int com_close_socket(int d);
int com_send_socket_message(struct socket_output *o);
int com_get_socket_message(struct socket_input  *i);
char * com_get_service_name(int d);
void com_create_fifo_name(char *s,char *m);
int com_open_fifo(char *s,enum wdx_modes  h);
int com_close_fifo(int d);
void com_cancel_fifo(int d);
int com_send_fifo_a_message(struct fifo_output *o);

/* DM 14/09/03 replace 'ftime' with 'gettimeofday()' */
/*
int com_select(int d,struct timeb *t);
*/

int com_select(int d,struct timeval *t);

int com_send_fifo_s_message(struct fifo_output *o);
int com_get_fifo_a_message(struct fifo_input  *i);
int com_get_fifo_s_message(struct fifo_input  *i);

/* kernel routines */
void ker_init(enum startup_mode s,struct group *g,struct message *m);
int  ker_scheduler();
void ker_build_acknowledge(pid_t p,enum process_status s,int i,char *c);
void ker_build_control(int i,enum process_status s);
void ker_build_notification(pid_t p, enum process_status s,int i, int g, enum resource_status r, char *d);
void ker_clean_up(pid_t pid);

/* watchdog routines */
void wat_init(enum startup_mode s,struct group *g,struct message *m);
void wat_threshold();
void wat_analyze_networks();
int  wat_input(struct watchdog *w,int a);
void wat_activity_set(enum mask lockvalue,enum mask scriptvalue);
void wat_validate_networks(int decrement);
int  wat_local_member_transition(enum watchdog_status stat, int *brdflgpt);
int  wat_remote_member_transition();
void wat_failure(enum watchdog_status stat,int *brdflgpt);
int  wat_node_exception(char *a,char *s);

/* specific routines */
char *spc_get_process_name(char *val);
struct mission_list *spc_get_running_mission();
int spc_path_init(char **p);
void spc_rename(char *log);
int spc_unlink(char *f);
int spc_priority_funct(int *c,char *w);
int spc_strcasecmp(char *s1,char *s2);
int spc_probe_license();
void spc_release_license(void);
int spc_check_mount_point(char *mpoint,enum wdx_modes md);
int spc_create_daemon(char *mis,struct group *gp,struct message *m,int (* scheduler)());
void spc_trigger_script(char *p,char *m);
int spc_open_socket(WDX_SOCKET *s,int af,int type,int proto);
int spc_close_socket(WDX_SOCKET s);
int spc_bind_socket(WDX_SOCKET s,struct sockaddr *name,int namelen);
int spc_setopt_socket(WDX_SOCKET s,int level,int optname,char *optval,int optlen);
int spc_getopt_socket(WDX_SOCKET s,int level,int optname, char *optval,int *optlen);
int spc_select_socket(WDX_SOCKET s,struct timeval *timeout);
int spc_write_socket(WDX_SOCKET s,char *b,int bl,int f,struct sockaddr *a,int al);
int spc_read_socket(WDX_SOCKET s,char *b,int bl, int f,struct sockaddr *a,int *al);
int spc_get_ifconfig (char *ifname,struct in_addr *ifaddr,struct in_addr *netmask,struct in_addr *brdaddr);
int spc_open_fifo(WDX_FIFO *f,char *fifoname,int size,int nbmess,enum wdx_modes mode);
int spc_close_fifo(WDX_FIFO f,char *fifoname,enum wdx_modes mode);
void spc_cancel_fifo(WDX_FIFO f);
int spc_write_fifo(WDX_FIFO f,char *buffer,int length);
int spc_read_fifo(WDX_FIFO f,char *buffer,int length);
int spc_process_running(pid_t pid);
int spc_super_user();
int spc_help(FILE *logdes);
int spc_set_lock(char *s);
int spc_lookat_daemon_lock(char *s,void (*funct)(),int slot);
int spc_lookat_app_lock(char *s,void (*funct)(),pid_t pid );
void spc_unlock(int lockid);
time_t spc_build_generation();
struct tm *spc_localtime(time_t *t);
void spc_communication_init();

/* structure for init module pointers : an init pointer is a function pointer
  used for linked module initialization. The arguments passed to this function
  are :
	- startup mode ( COLD_INIT, COLD_REINIT, WARM_REINIT )
	- pt to group structure
	- pt to message structure
*/
struct function_table {
void (* functpt)();
};

#ifdef WDX_LIBRARY_MODULE
#ifndef WDX_BUILD_API
struct function_table inittable[]={
	{ ker_init },
	{ lck_init },
	{ wat_init },
	{ 0 }
};
#endif /* WDX_BUILD_API */
#endif /* WDX_LIBRARY_MODULE */

#endif /* WDX_H */


