/*
 *   WDX: WitchDoctor-WDX, versatile distributed lock manager
 *   Copyright (C) 2003 Dominique Chabord,
 *   Please refer to credits section below
 *
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   ************************************************
 *
 *   History and credits:
 *   First version of WitchDoctor-WDX was based on:
 *
 *   DECwatchdog/Autopilot, watchdog and lock manager
 *   Copyright (C) 2003 HP
 *
 *   Developed by
 *     Computer Special Systems, Annecy France
 *     A Division of Digital Equipment Corporation
 *   and
 *     Custom-Systems, Annecy France
 *     An engineering center of Compaq
 *
 *   Provided under GNU-GPL terms by HP, 30 June 2003
 *   GNU-GPL replaces previous license statements
 *
 *   Rebranded, ported and modified for Shaman-X project, 15 July 2003
 *   by Dominique Chabord
 *   and maintained by other contributors as mentionned in the code
 *
 *   ************************************************
 *
 *   WDX is promoted by Shaman-X project
 *
 *   Check for new versions and documentation at
 *   http://www.shaman-x.org
 *
 *   ************************************************
 *
 *   Most of WDX code is designed to run unchanged on
 *   different operating systems, including at V1.0 time:
 *   Gnu-Linux, WindowsNT, DigitalUNIX and OpenVMS.
 *   Operating system specifics are identified by dedicated files.
 *
 *   Evolutions
 *   ----------
 *   July-2003     Dominique Chabord    WitchDoctor-WDX V1.0
 *                 Creation
 *
 *   ************************************************
 *
 *
 *	wdx_communication.c
 *
 *	Module including communication routines used by WDX
 *	Public routines follow the naming convention :
 *		com_xxxxxxx()
 *
 *
 ************************************************************************/
/*
 *
 *  06-Apr-94 -- Cornu
 *      Creation
 *  MKOO1 M.Kennealy  Changes made for intel porting. These should not affect
 *                    any other systems.
 */

#undef	WDX_COMMUNICATION_MODULE
#define WDX_COMMUNICATION_MODULE

#include	"wdx.h"



/*
 * COMMUNICATION INTERFACES definitions
 */

#define UDP "udp"

#define ALREADY_REG -1
#define NOT_REG 0
#define NO_SOCKET_FREE -1
#define NO_FIFO_FREE -1

#define FIFOS 2
#define FIFO_EXT 4
enum fifos_number { ASYN_FIFO, SYN_FIFO };


/*
 * Private data structure
 */

enum    inter_brdf  { XFALSE, XTRUE };

enum socket_use { SOCKET_NOT_USED, SOCKET_USED };

enum fifo_use { FIFO_NOT_USED, FIFO_USED };

/* Links description */
struct link
{
	char      ifname[IFNAMSIZ];	/* interface name */
	enum inter_use         use;	/* NA or USED */
	enum inter_brdf       brdf; /* reserved : NOT USED */
	struct in_addr      ifaddr; /* address of the interface */
	struct in_addr     netmask; /* net mask of the interface */
	struct in_addr     brdaddr; /* broadcast address of the interface */
};

/* FIFO description */
struct fifo
{
	char fifoname[FIFO_LEN+FIFO_EXT]; /* effective fifo name */
	enum fifo_use	         use; /* indicates that fifo opened/created */
	WDX_FIFO                     des; /* effective FIFO descriptor          */
	int				size; /* message size */
	int			      number; /* number of messages in fifo */
};

/* socket_descriptor includes all the data usefull for communicating over
 one socket interface */
struct socket_descriptor
{
	enum socket_use	      use;	/* slot use flag */
	WDX_SOCKET                des;	/* effective socket descriptor */
	enum wdx_modes           mode;	/* process or application */
	char     service[MISSION_LEN];      /* service name */
	int                redundancy;	/* redundancy value */
	struct  link  links[NETWORKS];	/* link table */
	struct sockaddr_in        dst;	/* socket address of interface	*/
	struct sockaddr_in        dae;      /* socket address of daemon     */
};

/* fifo_descriptor includes all the data usefull for communicating over
 one fifo interfaces */
struct fifo_descriptor
{
	enum fifo_use	      use;	/* slot use flag */
	char   globalfifo[FIFO_LEN+1];      /* fifo name */
	enum wdx_modes           mode;	/* process or application */
	struct fifo      fifos[FIFOS];	/* fifos table */
};

/*
 * Data declaration
 */
struct socket_descriptor socket_table[MAX_SOCKET];
struct fifo_descriptor  fifo_table[MAX_FIFO];
char fifo_extension[FIFOS][FIFO_EXT+1] = { "asy", "syn" };
int communicationinit = 0;

/*
 * Routines declaration
 */
/*char *inet_ntoa(); MK001 I don't think this is used anywhere */
int find_free_service();
int find_service_index(char *s);
int find_free_fifo();
int find_fifo_index(char *s);
unsigned int get_brdaddr(struct link *in);
unsigned int get_ifaddr(struct link *in);
int send_fifo_message(struct fifo_output *o,enum fifos_number    t);
int get_fifo_message(struct fifo_input  *i,enum fifos_number    t);
void communication_init();


/*
 * PUBLIC ROUTINES : prefixe is "com"
 */

/* int com_open_socket()
 * Set a socket environment.
 *
 * Argument :  service name
 *             redundancy
 *	       interfaces_used table
 *             WDX_PROCESS or WDX_APPLICATION
 *
 * Return   :  descriptor index if OK
 *            -1 if error
 */

int com_open_socket(s,r,i,h)
char                    *s;
int                      r;
struct interfaces_used  *i;
enum wdx_modes           h;
{
int _des;
int _d;
int _j;
int _l;
struct socket_descriptor *_pt;
struct interfaces_used *_iupt;
struct sockaddr_in       _sin; 		/* socket address of the sender	*/
struct servent *_sp;
struct hostent *_hp;
struct link    *_in;

int j;
struct in_addr daemon_ifaddr;
struct in_addr ifaddr;
struct in_addr netmask;
struct in_addr brdaddr;

/* communicationinit */
   if( !communicationinit )
   {
	   communication_init();
	   communicationinit++;
   }
/* check service param */
   if ( strlen(s) >= MISSION_LEN )
   {
	sprintf(messagestring,"Service name (%s) too long",s);
		PERROR_B
		messagestring
		PERROR_E;
		return(-1);
   }
/* check mode param */
   switch(h)
   {
   case WDX_APPLICATION :
   case WDX_PROCESS:
		break;
   default:
	sprintf(messagestring,"com_open_socket(), unknown mode=%d",h);
		PWARNING_B
		messagestring
		PWARNING_E;
		return(-1);
	break;
   }

/* search if service already registered */
   if ( find_service_index(s) == ALREADY_REG )
   {
	sprintf(messagestring,"Service (%s) already registered",s);
		PERROR_B
		messagestring
		PERROR_E;
		return(-1);
   }
/* search if free descriptor */
   if ( (_des = find_free_service()) == NO_SOCKET_FREE )
   {
	   PWARNING_B
	   "No socket free "
	   PWARNING_E;
	   return(-1);
   }

/* initialize data socket_descriptor structure */
   _pt = &socket_table[_des];
   _pt->use = SOCKET_USED;
   strncpy(_pt->service,s,MISSION_LEN);
   _pt->mode = h;
   _pt->redundancy = r;
   _iupt = i;

	for (_j = NETWORK_1; _j < NETWORKS; _j++)
	{
		strcpy (_pt->links[_j].ifname,_iupt->ifname);
		_pt->links[_j].use = _iupt->use;
		_iupt++;
	}

/* open socket */
   if ((spc_open_socket(&(_pt->des),AF_INET,SOCK_DGRAM,0)) < 0)
   {
	   PERROR_B
	   "Can't create an endpoint for communication %m"
	   PERROR_E;
	   _pt->use = SOCKET_NOT_USED;
	   return(-1);
   }

/* control on socket : CLOSE ON EXEC */
#ifdef WDX_ULTRIX
   if (fcntl(_pt->des,F_SETFD,FD_CLOEXEC) < 0)
   {
	   PERROR_B
	   "Can't close-on-exec at socket %m"
	   PERROR_E;
	   spc_close_socket(_pt->des);
	   _pt->use = SOCKET_NOT_USED;
	   return(-1);
   }
#endif /* WDX_ULTRIX */

/*#ifdef WDX_OSF1*/
/* modif lionel tricon */
   if (fcntl(_pt->des,F_SETFD,FD_CLOEXEC) < 0)
   {
	   PERROR_B
	   "Can't close-on-exec at socket %m"
	   PERROR_E;
	   spc_close_socket(_pt->des);
	   _pt->use = SOCKET_NOT_USED;
	   return(-1);
   }
/*#endif*/ /* WDX_OSF1 */

/* get port name */
   _sp = getservbyname(_pt->service,UDP);	/* port names are returned in 	*/
   if (_sp == NULL)		        /* network byte order !		*/
   {
	   sprintf(messagestring,"%s/%s is an unknown service",s,UDP);
	   PERROR_B
	   messagestring
	   PERROR_E;
	   spc_close_socket(_pt->des);
	   _pt->use = SOCKET_NOT_USED;
	   return(-1);
   }

/* get localhost name */
   _hp = gethostbyname("localhost");	/* needed by client		*/

	if (h == WDX_APPLICATION)			/* get the IP address that the */
	{									/* daemon process is using */
		for (j = 0; j < NETWORKS; j++)	/* for each network interface */
		{
			if (_pt->links[j].use == USED )	/* see if interface is used */
			{
				_in = &_pt->links[j];
				if ( spc_get_ifconfig(_in->ifname,&daemon_ifaddr,
										  &netmask,&brdaddr) == -1 )
				{
					spc_close_socket(_pt->des);
					_pt->use = SOCKET_NOT_USED;
					return(-1);
				}
				j = NETWORKS;
	 		}
		}
	}

   if (_hp == NULL)
   {
	   PERROR_B
	   "localhost is an unknown host"
	   PERROR_E;
	   spc_close_socket(_pt->des);
	   _pt->use = SOCKET_NOT_USED;
	   return(-1);
   }

/* define IP source address */
   memset((char *) &_sin, '\0', sizeof(_sin));	/* define IP source address */
   _sin.sin_family	= AF_INET;

	if (using_ip_addresses)		/* If using IP addresses then get the
								/* IP address of the interface we are using */
	{
		if ( h == WDX_PROCESS )
   		{
			for (j = 0; j < NETWORKS; j++)	/* for each network interface */
				if (_pt->links[j].use == USED )	/* see if interface is used */
				{
					_in = &_pt->links[j];
					if ( spc_get_ifconfig(_in->ifname,&ifaddr,
												  &netmask,&brdaddr) == -1 )
					{
						spc_close_socket(_pt->des);
						_pt->use = SOCKET_NOT_USED;
						return(-1);
					}
					j = NETWORKS;
			 	}
			_sin.sin_addr = ifaddr;
   		}
	}
	else
		_sin.sin_addr.s_addr	= htonl(INADDR_ANY);	/* else any address */

   if ( h == WDX_PROCESS )
		_sin.sin_port = _sp->s_port;
   else _sin.sin_port = 0;

   if (spc_bind_socket(_pt->des,(struct sockaddr *)&_sin,sizeof(_sin)) < 0)
   {
	   PERROR_B
	   "Can't bind a name to the socket %m"
	   PERROR_E;
	   spc_close_socket(_pt->des);
	   _pt->use = SOCKET_NOT_USED;
	   return(-1);
   }

/* define broadcast address */
/* IP dst addr for broadcasts on network */
   memset((char *) &(_pt->dst), '\0', sizeof(struct sockaddr_in));
   _pt->dst.sin_family	= AF_INET;
   _pt->dst.sin_port 	= _sp->s_port;

/* define broadcast address */
/* IP dest address for comm  with local WDX_PROCESS */
   memset((char *) &(_pt->dae), '\0', sizeof(struct sockaddr_in));
   _pt->dae.sin_family	= AF_INET;
   _pt->dae.sin_port 	= _sp->s_port;

   switch (h)
   {
   case WDX_PROCESS :
//printf("case WDX_PROCESS\n");
	_d = 1;
	if (spc_setopt_socket(_pt->des,
			   SOL_SOCKET,
			   SO_BROADCAST,
			   (char *)&_d,
			   sizeof(_d)) < 0)
		{
			PERROR_B
			"Can't set SO_BROADCAST on socket %m"
			PERROR_E;
		spc_close_socket(_pt->des);
			_pt->use = SOCKET_NOT_USED;
		return(-1);
		}
	if (spc_setopt_socket(_pt->des,
			   SOL_SOCKET,
			   SO_DONTROUTE,
			   (char *)&_d,
			   sizeof(_d)) < 0)
		{
			PERROR_B
			"Can't set SO_DONTROUTE on socket %m"
			PERROR_E;
		spc_close_socket(_pt->des);
			_pt->use = SOCKET_NOT_USED;
		return(-1);
		}

	/* check if udp broadcast is really enabled */
	_d = 0;
	_l = sizeof(_d);
	if (spc_getopt_socket(_pt->des,
			   SOL_SOCKET,
			   SO_BROADCAST,
			   (char *)&_d,
			   &_l) < 0)
		{
			PERROR_B
			"Can't get options on socket %m"
			PERROR_E;
		spc_close_socket(_pt->des);
			_pt->use = SOCKET_NOT_USED;
		return(-1);
		}
	if (!_d)
		{
			PERROR_B
			"Can't enable broadcast on udp"
			PERROR_E;
		spc_close_socket(_pt->des);
			_pt->use = SOCKET_NOT_USED;
		return(-1);
		}

		/* daemon init */
		for(_d=0;_d<NETWORKS;_d++)
			if(_pt->links[_d].use == USED )
			 {
			_in = &_pt->links[_d];
//printf("coicoin1\n");
					if ( spc_get_ifconfig(_in->ifname,
					  &_in->ifaddr,
					  &_in->netmask,
									  &_in->brdaddr) == -1 )
			{
//printf("BAD coicoin1\n");
/*printf("(%s) (%s) (%s)\n", _in->ifaddr, _in->netmask, _in->brdaddr);*/
						 spc_close_socket(_pt->des);
						 _pt->use = SOCKET_NOT_USED;
						 return(-1);
					}
//printf("GOOD coicoin1\n");
/*printf("(%s) (%s) (%s)\n", _in->ifaddr, _in->netmask, _in->brdaddr);*/
			 }
	break;

   case WDX_APPLICATION :
		if (using_ip_addresses)		/* If using IP addresses */
			_pt->dae.sin_addr = daemon_ifaddr;
		else
			memcpy((char *)&(_pt->dae.sin_addr.s_addr),_hp->h_addr,_hp->h_length);

		/* no redundancy required for localhost messages */
		_pt->redundancy = 1;
		break;
   }

   return (_des);
}

/* int com_update_socket()
 * Reset a socket environment.
 *
 * Argument :  descriptor
 *             redundancy
 *	       interfaces_used table
 *
 * Return   :  descriptor index if OK
 *            -1 if error
 */

int com_update_socket(d,r,i)
int			 d;
int                      r;
struct interfaces_used  *i;
{
int _j;
struct socket_descriptor *_pt;
struct interfaces_used *_iupt;
struct link              *_in;
/* communicationinit */
   if( !communicationinit )
   {
	   communication_init();
	   communicationinit++;
   }
   /* check if valid descriptor */
   if ( (d < MAX_SOCKET) && (d >= 0) )
   {
		/* check if used descriptor */
		if ( socket_table[d].use == SOCKET_USED )
		{

			 /* initialize data socket_descriptor structure */
			 _pt = &socket_table[d];
			 _pt->use = SOCKET_USED;
			 _pt->redundancy = r;
			 _iupt = i;
			 for(_j=NETWORK_1;_j<NETWORKS;_j++)
			 {
				 strcpy(_pt->links[_j].ifname,_iupt->ifname);
				 _pt->links[_j].use = _iupt->use;
				 _iupt++;
			 }
			 for(_j=0;_j<NETWORKS;_j++)
				 if(_pt->links[_j].use == USED )
			 {
			_in = &_pt->links[_j];
//printf("coicoin2\n");
					if ( spc_get_ifconfig(_in->ifname,
					  &_in->ifaddr,
					  &_in->netmask,
									  &_in->brdaddr) == -1 )
			{
						 spc_close_socket(_pt->des);
						 _pt->use = SOCKET_NOT_USED;
						 return(-1);
					}
			 }
	   }
   }
   return (d);
}

/* void com_close_socket()
 * Close the communication socket environment
 *
 * Argument : descriptor
 *
 * Return   : None
 *
 */
int com_close_socket(d)
int d;
{
/* communicationinit */
   if( !communicationinit )
   {
	   communication_init();
	   communicationinit++;
   }
/* check if valid descriptor */
	if ( (d>= MAX_SOCKET) || (d<0))
	return(-1);

/* check if used descriptor */
	if ( socket_table[d].use == SOCKET_USED )
		 spc_close_socket(socket_table[d].des);
	socket_table[d].use = SOCKET_NOT_USED;
	return(0);
}

/* int com_send_socket_message()
 *
 * Argument : pointer to output data structure
 *
 * Global   : None
 *
 * Return   :  0 OK
 *            -1 error
 */
int com_send_socket_message(o)
struct socket_output *o;	/* output data structure */
{
struct socket_descriptor *_p;   /* pointer to socket_descriptor */
int                _msgcount;   /* message count */
enum networks            _i;    /* network number */
int                     _ret;   /* returned value */
int i,j;
unsigned int ifaddr;

/* communicationinit */
   if( !communicationinit )
   {
	   communication_init();
	   communicationinit++;
   }
   _ret = -1;
   /* check if o NULL */
   if ( o )
   {
	   /* check if valid descriptor */
	   if ( (o->descriptor < MAX_SOCKET) && (o->descriptor >= 0) )
	   {
			/* check if size valid */
		   if ( o->size <= WDX_SOCKET_DATA_SIZE )
		   {
				/* check if used descriptor */
				if ( socket_table[o->descriptor].use == SOCKET_USED )
				{
					 _p = &(socket_table[o->descriptor]);
			 _ret = 0;
				}
		   }
	   }
	   if ( _ret == -1 )
	   {
		sprintf(messagestring,
			"Invalid socket communication descriptor (%d)",
			o->descriptor);
		PERROR_B
		messagestring
		PERROR_E;
		return(_ret);
	   }
	   _msgcount = _p->redundancy;
	   while (_msgcount-- > 0)
	   {
		   if (_p->mode == WDX_PROCESS )
		   {
			  for(_i=NETWORK_1;_i<NETWORKS;_i++)	/* for each network */
				  if ( _p->links[_i].use == USED )	/* if interface is used */
				  {
					if (using_ip_addresses) /* send to each IP address */
					{
									/* send to ours first */
						ifaddr = get_ifaddr (&(_p->links[_i]));
				   		_p->dst.sin_addr.s_addr = ifaddr;
                   		if ( spc_write_socket(_p->des, o->buffer, o->size,
					    	   0, (struct sockaddr *)&(_p->dst),
                                  sizeof(struct sockaddr_in))
  	                               != 0 )
   		               	{
       		               	sprintf(messagestring,
           		           	"Write socket failed on %s",_p->links[_i].ifname);
               		       	PWARNING_B
							messagestring
							PWARNING_E;
                           	_ret = -1;
               			}

						for (j = 0; j < MEMBER_COUNT; j++)
						{				/* now for the other addresses */
					   		if ((members_ip[j].s_addr != 0) &&
									 (members_ip[j].s_addr != ifaddr))
					   		{
						   		_p->dst.sin_addr.s_addr = members_ip[j].s_addr;
    	                   		if ( spc_write_socket(_p->des, o->buffer, o->size,
							    	   0, (struct sockaddr *)&(_p->dst),
	                                   sizeof(struct sockaddr_in))
    	                               != 0 )
        		               	{
            		               	sprintf(messagestring,
                		           	"Write socket failed on %s",_p->links[_i].ifname);
                    		       	PWARNING_B
									messagestring
									PWARNING_E;
	                            	_ret = -1;
                       			}
					   		}
						}
					}
						/* always send a broadcast message */
					   _p->dst.sin_addr.s_addr = get_brdaddr(&(_p->links[_i]));
					   if ( spc_write_socket(_p->des,
								   o->buffer,
							   o->size,
								   0,
								   (struct sockaddr *)&(_p->dst),
								   sizeof(struct sockaddr_in))
								   != 0 )
					   {
							sprintf(messagestring,
							"Write socket failed on %s",_p->links[_i].ifname);
							PWARNING_B
							messagestring
							PWARNING_E;
							_ret = -1;
					   }
				  }
		   }
		   else				/* if not WDX_PROCESS */
		   {
			   if ( spc_write_socket(_p->des,o->buffer,o->size,0,
						  (struct sockaddr *)&(_p->dae),
			  sizeof(struct sockaddr_in))
						  != 0 )
			   {
/*++ 30-May-94 BC : comments */
/*                    PWARNING_B
/*                    "Write socket failed to wdx process"
/*                    PWARNING_E;
/*-- 30-May-94 BC */
					_ret = -1;
			   }
		   }
	   }
   }
   return(_ret);
}


/* int com_select()
 * Select a message on the descriptor specified until timeout
 *
 *
 * Argument : communication descriptor
 *	      timeb struct pt
 *
 * Return   :
 *
 */
int com_select(int d, struct timeval *t)
{
struct timeval timeout;

/* communicationinit */
   if( !communicationinit )
   {
	   communication_init();
	   communicationinit++;
   }

	if ( (d>= MAX_SOCKET) || (d<0))
		return(-1);

/* check if used descriptor */
	if ( socket_table[d].use != SOCKET_USED )
		return(-1);

/* DM 14/09/03 replace 'ftime' with 'gettimeofday()' */
/*
	timeout.tv_sec = (long)t->time;
	timeout.tv_usec = milli_to_micro(((long)t->millitm));
*/
	timeout = *t;

/* valid descriptor : call specific socket routine */

	return(spc_select_socket(socket_table[d].des,&timeout));
}

/* int com_get_socket_message()
 * Read a message from socket defined by its descriptor
 *
 * Argument : pointer to input data structure
 *
 * Global   : None
 *
 * Return   :  0 OK
 *            -1 invalid message read or error.
 */
int com_get_socket_message(i)
struct socket_input  *i;
{
struct socket_descriptor *_p;   /* pointer to socket_descriptor */
struct sockaddr_in     _addr;   /* to store input interface address */
int			_ret;   /* returned value */
int                    _lfcv;   /* length in bytes of the fcv buffer */
int                   _found;   /* flag to get network number */
int                       _j;   /* network number index */
struct in_addr          _rcv;   /* received address */
struct in_addr          _ifa;   /* interface address */

/* communicationinit */
   if( !communicationinit )
   {
	   communication_init();
	   communicationinit++;
   }

   _ret = -1;
   if ( i )
   {
	   /* check if valid descriptor */
	   if ( (i->descriptor < MAX_SOCKET) && (i->descriptor >= 0))
	   {
			/* check if used descriptor */
			if ( socket_table[i->descriptor].use == SOCKET_USED )
			{
				 _p = &(socket_table[i->descriptor]);
				 /* check buffer size */
				 if ( i->size == WDX_SOCKET_DATA_SIZE )
				 {
					 _lfcv = sizeof(_addr);
					 if ( spc_read_socket(_p->des,
								   i->buffer,
								   i->size,
								   0,
								   (struct sockaddr *)&(_addr),
									&_lfcv) == 0 )
				  _ret = 0;
				 }
			}
	   }
	   if ( _ret == -1 )
	   {
			PWARNING_B
			"Read socket failed %m"
			PWARNING_E;
			return(_ret);
	   }

/* network decapsulation */
	   _found = 0;
	   _j = 0;
	   while( (_j<NETWORKS) && (!_found) )
	   {
		   _rcv.s_addr = _addr.sin_addr.s_addr & _p->links[_j].netmask.s_addr;
		   _ifa.s_addr = _p->links[_j].ifaddr.s_addr &
						_p->links[_j].netmask.s_addr;
		   if ( (_rcv.s_addr == _ifa.s_addr) && (_p->links[_j].use == USED) )
				_found = 1;
		   else _j++;
	   }
	   if ( _found )
	   {
		i->number = _j;
			_ret = 0;
	   }
	   else
	   {    i->number = -1;
			_ret = -1;
	   }
   }
   return(_ret);
}

/* char * com_get_service_name()
 * For a valid communication descriptor, return the UDP service name
 *
 * Argument : descriptor
 *
 * Return : pt to service name / NULL if descriptor is invalid
 */
char * com_get_service_name(int d)
{
/* communicationinit */
   if( !communicationinit )
   {
	   communication_init();
	   communicationinit++;
   }
	if ( (d>= MAX_SOCKET) || (d<0))
	return(NULL);

/* check if used descriptor */
	if ( socket_table[d].use != SOCKET_USED )
	return(NULL);

/* return service value */
	return(socket_table[d].service);
}

/* void com_create_fifo_name()
 * Create a WDX fifo name
 *
 * Argument : pt to string to fill in
 *	      missionname pt
 *
 * Return   : None
 *
 */
void com_create_fifo_name(s,m)
char *s;
char *m;
{
/* communicationinit */
   if( !communicationinit )
   {
	   communication_init();
	   communicationinit++;
   }

 if ( s && m )
   sprintf(s,"%s%d",m,getpid());
}

/* int com_open_fifo()
 * Open a fifo environment
 *
 * Argument :  global fifo name
 *             WDX_PROCESS or WDX_APPLICATION
 *
 * Return   :  descriptor index if OK
 *            -1 if error
 */

int com_open_fifo(s,h)
char                    *s; /* fifo name */
enum wdx_modes           h; /* creation mode */
{
int _des;
int _j;
struct fifo_descriptor *_pt;
struct fifo        *_fifopt;


/* communicationinit */
   if( !communicationinit )
   {
	   communication_init();
	   communicationinit++;
   }

/* check if global fifo name too long */
   if ( strlen(s) > FIFO_LEN )
	return(-1);
/* check mode param */
   switch(h)
   {
   case WDX_APPLICATION :
   case WDX_PROCESS:
		break;
   default:
	sprintf(messagestring,"open_fifo(), unknown mode=%d",h);
		PWARNING_B
		messagestring
		PWARNING_E;
		return(-1);
	break;
   }

/* search if global fifo already created */
   if ( find_fifo_index(s) == ALREADY_REG )
	   return(-1);

/* search if free descriptor */
   if ( (_des = find_free_fifo()) == NO_FIFO_FREE )
	   return(-1);

/* initialize data fifo_descriptor structure */


   _pt = &fifo_table[_des];
   _pt->use = FIFO_USED;
   strncpy(_pt->globalfifo,s,FIFO_LEN);
   _pt->mode = h;

   _fifopt = &(_pt->fifos[0]);
   for(_j=ASYN_FIFO;_j<FIFOS;_j++)
   {
	  sprintf(_fifopt->fifoname,"%s%s",_pt->globalfifo,fifo_extension[_j]);
	  if ( _j == ASYN_FIFO )
	  {
	   _fifopt->size = sizeof(struct process_message);
	   _fifopt->number = ASYN_NUMBER_OF_MESSAGES;
	  }
	  else
	  {
	   _fifopt->size = sizeof(struct process_acknowledge);
	   _fifopt->number = SYN_NUMBER_OF_MESSAGES;
	  }
	  if ( spc_open_fifo(&(_fifopt->des),
			   _fifopt->fifoname,
			   _fifopt->size,
			   _fifopt->number,
			   _pt->mode) == -1 )
	  {

	   sprintf(messagestring,"Can't open FIFO %s %%m",_fifopt->fifoname);
	   PERROR_B
	   messagestring
	   PERROR_E;
	   for(_j=ASYN_FIFO;_j<FIFOS;_j++)
	   {
			if( _pt->fifos[_j].use == FIFO_USED )
		{
			spc_close_fifo(_pt->fifos[_j].des,
						   _pt->fifos[_j].fifoname,
				   h);
			_pt->fifos[_j].use = FIFO_NOT_USED;
		}
	   }
	   _pt->use = FIFO_NOT_USED;
	   return(-1);
	  }
	  _fifopt->use = FIFO_USED;
	  _fifopt++;
   }

   return (_des);
}

/* int com_close_fifo()
 * Close the fifo communication environment
 *
 * Argument : descriptor
 *
 * Return   : None
 *
 */
int com_close_fifo(d)
int d;
{
int _i;

/* communicationinit */
   if( !communicationinit )
   {
	   communication_init();
	   communicationinit++;
   }

/* check if valid descriptor */
	if ( (d>= MAX_FIFO) || (d<0))
	return(-1);

/* check if used descriptor */
	else
	{
		if ( fifo_table[d].use == FIFO_USED )
	{
		for(_i=ASYN_FIFO;_i<FIFOS;_i++)
		{
				if ( fifo_table[d].fifos[_i].use == FIFO_USED )
		{
			 spc_close_fifo(fifo_table[d].fifos[_i].des,
						fifo_table[d].fifos[_i].fifoname,
							fifo_table[d].mode);
				 fifo_table[d].fifos[_i].use = FIFO_NOT_USED;
		}
			 }
		}
		fifo_table[d].use = FIFO_NOT_USED;
	}
	return(0);
}

/* int com_cancel_fifo()
 * Cancel fifo communication pending IO
 *
 * Argument : descriptor
 *
 * Return   : None
 *
 */
void com_cancel_fifo(d)
int d;
{
int _i;

/* communicationinit */
   if( !communicationinit )
   {
	   communication_init();
	   communicationinit++;
   }

/* check if valid descriptor */
	if ( (d< MAX_FIFO) && (d>=0))
	{

/* check if used descriptor */
		if ( fifo_table[d].use == FIFO_USED )
	{
/* check if used fifo : cancel IO */
		for(_i=ASYN_FIFO;_i<FIFOS;_i++)
				if ( fifo_table[d].fifos[_i].use == FIFO_USED )
			 spc_cancel_fifo(fifo_table[d].fifos[_i].des);
		}
	}
}

/* int com_send_fifo_a_message()
 * Send a message through an asynchronous FIFO defined by descriptor
 *
 * int com_send_fifo_s_message()
 * Send a message through a synchronous FIFO defined by descriptor
 *
 * Argument : pointer to fifo_output data structure
 *
 * Global   : None
 *
 * Return   :  0 OK
 *            -1 error
 */
int com_send_fifo_a_message(o)
struct fifo_output *o;		/* output data structure */
{

/* communicationinit */
   if( !communicationinit )
   {
	   communication_init();
	   communicationinit++;
   }

   return(send_fifo_message(o,ASYN_FIFO));
}

int com_send_fifo_s_message(o)
struct fifo_output *o;		/* output data structure */
{

/* communicationinit */
   if( !communicationinit )
   {
	   communication_init();
	   communicationinit++;
   }

   return(send_fifo_message(o,SYN_FIFO));
}
/* int com_get_fifo_a_message()
 * Read a message from an asynchronous FIFO defined by its descriptor
 *
 * int com_get_fifo_s_message()
 * Read a message from a synchronous FIFO defined by its descriptor
 *
 * Argument : pointer to fifo_input data structure
 *
 * Global   : None
 *
 * Return   : number of bytes read
 *            -1 invalid message read or error.
 */
int com_get_fifo_a_message(i)
struct fifo_input  *i;
{

/* communicationinit */
   if( !communicationinit )
   {
	   communication_init();
	   communicationinit++;
   }

   return(get_fifo_message(i,ASYN_FIFO));
}

int com_get_fifo_s_message(i)
struct fifo_input  *i;
{

/* communicationinit */
   if( !communicationinit )
   {
	   communication_init();
	   communicationinit++;
   }

   return(get_fifo_message(i,SYN_FIFO));
}

/*
 * LOCAL ROUTINES
 */

/* int find_free_service()
 * Search for the first free index in the socket_table
 *
 * Argument : None
 *
 * Return   : index if OK
 *	      NO_SOCKET_FREE if table full
 */
int find_free_service()
{
int   _i;    /* index in table */
int _ret;    /* returned value */

	_ret = NO_SOCKET_FREE;
	for(_i=0;_i<MAX_SOCKET;_i++)
	{
	if ( socket_table[_i].use == SOCKET_NOT_USED )
	{
		_ret = _i;
		break;
	}
	}
	return(_ret);
}

/* int find_service_index()
 * Search if a service is already registered
 *
 * Argument : Pointer to service name
 *
 * Return   : ALREADY_REG if already registered
 *	      NOT_REG if not registered
 */
int find_service_index(s)
char *s;
{
int   _i;    /* index in table */
int _ret;    /* returned value */

	_ret = NOT_REG;
	for(_i=0;_i<MAX_SOCKET;_i++)
	{
	if ( socket_table[_i].use == SOCKET_USED )
	{
		 if ( strcmp(socket_table[_i].service,s) == 0 )
		 {
			  _ret = ALREADY_REG;
			  break;
		 }
	}
	}
	return(_ret);
}

/* int find_free_fifo()
 * Search for the first free index in the fifo_table
 *
 * Argument : None
 *
 * Return   : index if OK
 *	      NO_FIFO_FREE if table full
 */
int find_free_fifo()
{
int   _i;    /* index in table */
int _ret;    /* returned value */

	_ret = NO_FIFO_FREE;
	for(_i=0;_i<MAX_FIFO;_i++)
	{
	if ( fifo_table[_i].use == FIFO_NOT_USED )
	{
		_ret = _i;
		break;
	}
	}
	return(_ret);
}

/* int find_fifo_index()
 * Search if a fifo is already registered
 *
 * Argument : Pointer to global fifo name
 *
 * Return   : ALREADY_REG if already registered
 *	      NOT_REG if not registered
 */
int find_fifo_index(s)
char *s;
{
int   _i;    /* index in table */
int _ret;    /* returned value */

	_ret = NOT_REG;
	for(_i=0;_i<MAX_FIFO;_i++)
	{
	if ( fifo_table[_i].use == FIFO_USED )
	{
		 if ( strcmp(fifo_table[_i].globalfifo,s) == 0 )
		 {
			  _ret = ALREADY_REG;
			  break;
		 }
	}
	}
	return(_ret);
}

/* unsigned int get_brdaddr()
 * return broadcast address of ethernet device name
 */
unsigned int get_brdaddr(in)
struct link *in;
{
//printf("coicoin3\n");
 if ( spc_get_ifconfig(in->ifname,
					 &(in->ifaddr),
					 &(in->netmask),
					 &(in->brdaddr)) == 0)
	  return ( in->brdaddr.s_addr );
 return( (unsigned int)0);
}

/* unsigned int get_ifaddr()
 * return internet address of ethernet device name
 */
unsigned int get_ifaddr(in)
struct link *in;
{
	if ( spc_get_ifconfig(in->ifname,
                     &(in->ifaddr),
                     &(in->netmask),
                     &(in->brdaddr)) == 0)
		return ( in->ifaddr.s_addr );
	return( (unsigned int)0);
}

/* int send_fifo_message()
 * Send a message through a FIFO defined by descriptor and fifo number
 *
 * Argument : pointer to fifo_output data structure
 *	      fifo number
 *
 * Global   : None
 *
 * Return   :  0 OK
 *            -1 error
 *	      -2 pipe full
 */
int send_fifo_message(o,t)
struct fifo_output *o;
enum fifos_number    t;
{
struct fifo_descriptor *_p;   /* pointer to fifo_descriptor */
int                   _ret;   /* returned value */

   _ret = -1;
/* check if o NULL */
   if ( o )
   {
	   /* check if valid descriptor */
	   if ( (o->descriptor < MAX_FIFO) &&
			(o->descriptor >= 0) )
	   {
		/* check if used descriptor */
			if ( fifo_table[o->descriptor].use == FIFO_USED &&
				 fifo_table[o->descriptor].fifos[t].use == FIFO_USED )
			{
				 _p = &(fifo_table[o->descriptor]);

			 /* send message */
		 _ret = spc_write_fifo(_p->fifos[t].des,o->buffer,o->size);
			}
	   }
	   if ( _ret == -1 )
	   {
			PWARNING_B
			"Write to FIFO failed"
			PWARNING_E;
	   }
   }
   return(_ret);
}

/* int get_fifo_message()
 * Read a message from the FIFO defined by its fifo number
 *
 * Argument : pointer to fifo_input data structure
 *	      fifo number
 *
 * Global   : None
 *
 * Return   : number of bytes read
 *            -1 invalid message read or error.
 */
int get_fifo_message(i,t)
struct fifo_input  *i;
enum fifos_number    t;
{
struct fifo_descriptor  *_p;   /* pt to current fifo_descriptor */
int		       _ret;   /* returned value */

   _ret = -1;
   if ( i )
   {
	   /* check if valid descriptor */
	   if ( (i->descriptor < MAX_FIFO) && (i->descriptor >= 0))
	   {
			/* check if used descriptor */
			if ( fifo_table[i->descriptor].use == FIFO_USED &&
						fifo_table[i->descriptor].fifos[t].use == FIFO_USED )
			{
				 _p = &(fifo_table[i->descriptor]);
				 _ret = spc_read_fifo(_p->fifos[t].des, i->buffer, i->size);
			}
	   }
   }
   return(_ret);
}

/* void communication_init()
 * Initialize module data structure
 *
 *
 */
void communication_init()
{
int _i;
int _j;

   for(_i=0;_i<MAX_SOCKET;_i++)
	socket_table[_i].use = SOCKET_NOT_USED;
   for(_i=0;_i<MAX_FIFO;_i++)
   {
	fifo_table[_i].use = FIFO_NOT_USED;
	for(_j=0;_j<FIFOS;_j++)
		fifo_table[_i].fifos[_j].use = FIFO_NOT_USED;
   }
   spc_communication_init();
}

