/*
 *   WDX: WitchDoctor-WDX, versatile distributed lock manager
 *   Copyright (C) 2003 Dominique Chabord,
 *   Please refer to credits section below
 *
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   ************************************************
 *
 *   History and credits:
 *   First version of WitchDoctor-WDX was based on:
 *
 *   DECwatchdog/Autopilot, watchdog and lock manager
 *   Copyright (C) 2003 HP
 *
 *   Developed by
 *     Computer Special Systems, Annecy France
 *     A Division of Digital Equipment Corporation
 *   and
 *     Custom-Systems, Annecy France
 *     An engineering center of Compaq
 *
 *   Provided under GNU-GPL terms by HP, 30 June 2003
 *   GNU-GPL replaces previous license statements
 *
 *   Rebranded, ported and modified for Shaman-X project, 15 July 2003
 *   by Dominique Chabord
 *   and maintained by other contributors as mentionned in the code
 *
 *   ************************************************
 *
 *   WDX is promoted by Shaman-X project
 *
 *   Check for new versions and documentation at
 *   http://www.shaman-x.org
 *
 *   ************************************************
 *
 *   Most of WDX code is designed to run unchanged on
 *   different operating systems, including at V1.0 time:
 *   Gnu-Linux, WindowsNT, DigitalUNIX and OpenVMS.
 *   Operating system specifics are identified by dedicated files.
 *
 *   Evolutions
 *   ----------
 *   July-2003     Dominique Chabord    WitchDoctor-WDX V1.0
 *                 Creation
 *
 *   ************************************************
 *
 *
 *	wdx_kernel.h
 *
 *	Kernel definitions, data structures, and constants
 *
 ************************************************************************/
/*
 *
 *  17-Feb-94 -- Cornu
 *      Creation
 *
 *  6-Feb-96  -- Ccile de Mare
 *      Removes a BUG (with PAUSE) : if ACPAUSETIMER in SUBSILENCE or
 *      SUBRUNNING sub-state, remain in the state
 */
#ifndef		WDX_KERNEL_H
#define		WDX_KERNEL_H


/*
 * the application_slot structure is used to store connection links state
 * between appplication and wdx process
 */

enum appl_slot { SLOT_NOT_USED, SLOT_USED };

enum trashed  { NOT_TRASHED, TRASHED };

enum protocol_event { NETWORK_EVENT,
		      TIMER_EVENT
		    };

struct application_slot
{
    enum appl_slot      use; /* use flag */
    pid_t               pid; /* id. of the application process */
    int	 	        des; /* fifo descriptor */
    enum control_mode  mode; /* api mode */
    enum trashed      trash; /* trashed messages flag */
    time_t         trashgen; /* generation number ot trash event */
};

struct application_lock
{
   pid_t   pid;		/* application pid */
   int    lock;		/* application lock */
};

/*
 * The command_wish structure is used to store all the information required
 * by the exclude protocol.
 */
struct command_wish
{
int                            index; /* local index */
int                             slot; /* slot in slot_table */
struct operation                oper; /* API message received */
enum command_request               r; /* request */
enum command_action                c; /* action */
enum command_answer  a[MEMBER_COUNT]; /* answer table */
time_t                    generation; /* request generation nb */
struct wdx_timer             silence; /* silence timer */
struct wdx_timer	     exclude; /* exclude timer */
struct wdx_timer               pause; /* pause timer */
struct group		      group1; /* group data structure : config read */
};


/* The datanumber structure is used to store a list of datanumber received
 * from each member. This is usefull to filter incoming messages.
 */
#define MAX_NUMBER       2	/* data number filter */

struct data_number
{
        unsigned int number[MAX_NUMBER];
        int index;
};

/*
 * USER table : refer to Functional Specification for table setting
 * Include for API operation value the action taken and action parameters.
 * The action depends on watchdog substatus :
 * ( SUBIDLE, SUBCOLD, SUBWARM, SUBSILENCE, SUBRUNNING, 
 * SUBSTOPPING )
 */

struct automate
{
int (* funct)();
enum watchdog_status       status; /* status */
enum watchdog_substatus substatus; /* substatus */
};

/* table describing what to do when an event occurs */
/* In function table : index values are :
 *	0 for SUBIDLE 
 *	1 for SUBCOLD
 *	2 for SUBWARM
 *	3 for SUBSILENCE
 *	4 for SUBRUNNING
 *	5 for SUBSTOPPING
 */
struct event_table
{
int                               value; /* operation value */
struct automate    evt[WATCH_SUBSTATUS]; /* function table */
int                          parameter1; /* command, lock, wait code */
};

/*
 * The request_action structure is defined for request to action conversion 
 */
struct request_action	/* conversion data structure */
{
int value;	/* request value */
int action;	/* associated action */
};

/* data structure used to build a queue of lock operation 
   delayed during silence time */
struct delayed_operation 
{
    struct delayed_operation *next;
    enum lock_request      request;
    struct operation          oper;
};

/* activity_protocol structure used to block protocols : 
 *  if YES is set the protocol is not blocked 
 *  if NO is set the protocol is blocked
 */
struct activity_protocol
{
    enum watchdog_substatus substatus; /* substatus value */
    void               (*lockfunct)(); /* lock activity update routine */
    void           (*watchdogfunct)(); /* watchdog activity update routine */
    enum mask		         lock; /* lock protocol mask value */
    enum mask		       script; /* script trigger mask */
};


#ifdef WDX_KERNEL_MODULE
/* Internal references */
/* void activity_algorithm(struct timeb *p,struct timeb *c,int e); */
void activity_algorithm(time_t *p, time_t *c,int e);
void activity_protocol(enum watchdog_substatus s);
void broadcast_test(time_t *p, time_t *c);
void reset_command_wish(struct command_wish *p);
unsigned int build_data_number();
int validate_data_number(int index,unsigned int datanumber);
void reset_application_slot(int d);
void reset_application_lock(int d);
int send_acknowledge(int d, enum process_status s,int i, char *c);
void create_asynchronous_message(int d,enum process_type y,enum process_status s, int i,int g,int r,char *t);
int find_application_slot(int id);
int is_checksum_valid(struct encoded_message *em);
int is_message_valid(struct message *m);
void validate_silence(int t);
void validate_pause(int t);
enum command_answer answer_to_start_request(int c);
void exclude_protocol(enum protocol_event e, int t);
void delete_delayed_operation(void);
int command_message(struct command *c);
int process_message(struct socket_input  *inpt);
int reject_by_prot(struct operation *o,int t);

/* Reference used in following automate tables */
int exclude_act(struct command *c, enum command_answer a);
int ignore_act(struct command *c, enum command_action a);
int conflict_act(struct command *c, enum command_request t);
int reject_act(struct command *c,enum command_request  t);
int accept_act(struct command *c, enum command_request a);
int process_act(struct command *c, enum command_action t);
int action_opr(struct operation *o,enum command_action a);
int aldone_opr(struct operation *o,enum command_action a);
int reject_opr(struct operation *o,int t);
int request_opr(struct operation *o,int t);
int local_opr(struct operation *o,int t);
int wait_opr(struct operation *c,int t);
int master_opr(struct operation *c,enum lock_request t);
int masidl_opr(struct operation *c,enum lock_request t);
int masdel_opr(struct operation *c,enum lock_request t);
int delay_opr(struct operation *c,enum lock_request t);
int lock_opr(struct operation *c,enum lock_request t);
int control_opr(struct operation *c,int t);


/* Local data definitions */
static struct application_slot applicationtable[MAX_APPLICATION_ON_PROCESS];
static struct application_lock locktable[MAX_APPLICATION_ON_PROCESS];
static struct data_number table[MEMBER_COUNT];  /* recevied message number table */
static unsigned int        localdt;    /* local data number stored */
static struct message        *mess;    /* pt to message structure */
static struct group           *gdp;    /* group pointer */
static struct member *localmemberp;    /* pt to local member definition */
struct socket_input          input;    /* input socket data structure */
struct encoded_message  in_message;    /* incomming watchdog message */
static struct delayed_operation *delayedroot = NULL;

static int        analyze; /* flag used to define if analyze is to be done */

/* used to define if the watchdog and lock protocol have to be blocked */
struct activity_protocol actprt[] =
{  { SUBIDLE,     
     lck_activity_set, wat_activity_set, NO, NO },
   { SUBCOLD, 
     lck_activity_set, wat_activity_set, NO, NO },
   { SUBWARM,  
     lck_activity_set, wat_activity_set, NO, NO },
   { SUBSILENCE, 
     lck_activity_set, wat_activity_set, NO, YES }, 
   { SUBRUNNING,
     lck_activity_set, wat_activity_set, YES, YES },
   { SUBSTOPPING,
     lck_activity_set, wat_activity_set, YES, YES },
   { SUBFAULT,
     lck_activity_set, wat_activity_set, YES, YES },
};

/* table used to convert an request value to an action value */
struct request_action reqact[] =
{
  { RRUN, ACRUN },
  { RSTOP, ACSTOP },
  { RSTART, ACSTART },
  { RPAUSE, ACPAUSE},
  { RPAUSESTART, ACPAUSESTART},
  { 0, 0 }
};


/* In function table : index values are :
 *	0 for SUBIDLE 
 *	1 for SUBCOLD
 *	2 for SUBWARM
 *	3 for SUBSILENCE
 *	4 for SUBRUNNING
 *	5 for SUBSTOPPING
 */
struct event_table opertab[] =
{
  { WDX_OPEN_CONTROL,		    /* open control operation */
    /* allowed in every substatus */
    {    { control_opr, W_IDLE,     SUBIDLE },
         { control_opr, W_IDLE,     SUBCOLD },
         { control_opr, W_IDLE,     SUBWARM },
         { control_opr, W_RUNNING,  SUBSILENCE },
         { control_opr, W_RUNNING,  SUBRUNNING },
         { control_opr, W_RUNNING,  SUBSTOPPING }
    },
    WDX_OPEN_CONTROL		    /* wdx_open_control parameter */
  },
  { WDX_CLOSE_CONTROL,		    /* close control operation */
    /* allowed in every substatus */
    {    { control_opr, W_IDLE,     SUBIDLE },
         { control_opr, W_IDLE,     SUBCOLD },
         { control_opr, W_IDLE,     SUBWARM },
         { control_opr, W_RUNNING,  SUBSILENCE },
         { control_opr, W_RUNNING,  SUBRUNNING },
         { control_opr, W_RUNNING,  SUBSTOPPING }
    },
    WDX_CLOSE_CONTROL               /* wdx_close_control parameter */
  },
  { WDX_GROUP_RUN,		    /* run api operation */
    /* allowed in every substatus */
    {    { request_opr, W_IDLE,     SUBCOLD },
         { reject_opr,  W_IDLE,     SUBCOLD },
         { reject_opr,  W_IDLE,     SUBWARM },
         { reject_opr,  W_RUNNING,  SUBSILENCE },
         { reject_opr,  W_RUNNING,  SUBRUNNING },
         { reject_opr,  W_RUNNING,  SUBSTOPPING }
    },
    RRUN		            /* RRUN request parameter */
  },
  { WDX_GROUP_KILL,		    /* kill api operation */
    /* allowed in every substatus */
    {    { action_opr, W_IDLE,     SUBIDLE },
         { action_opr, W_IDLE,     SUBCOLD },
         { action_opr, W_IDLE,     SUBWARM },
         { action_opr, W_RUNNING,  SUBSILENCE },
         { action_opr, W_RUNNING,  SUBRUNNING },
         { action_opr, W_RUNNING,  SUBSTOPPING }
    },
    ACKILL		            /* ACKILL action parameter */
  },
  { WDX_GROUP_STOP,		    /* stop api operation */
    {    { reject_opr,  W_IDLE,     SUBIDLE },
         { reject_opr,  W_IDLE,     SUBCOLD },
         { reject_opr,  W_IDLE,     SUBWARM },
         { request_opr, W_RUNNING,  SUBSTOPPING },
         { request_opr, W_RUNNING,  SUBSTOPPING },
         { reject_opr,  W_RUNNING,  SUBSTOPPING }
    },
    RSTOP		            /* RSTOP request parameter */
  },
  { WDX_GROUP_START,		    /* start api operation */
    {    { request_opr, W_IDLE,     SUBWARM },
         { reject_opr,  W_IDLE,     SUBCOLD },
         { reject_opr,  W_IDLE,     SUBWARM },
         { reject_opr,  W_RUNNING,  SUBSILENCE },
         { reject_opr,  W_RUNNING,  SUBRUNNING },
         { reject_opr,  W_RUNNING,  SUBSTOPPING }
    },
    RSTART		            /* RSTART request parameter */
  },
  { WDX_GROUP_PAUSE,		    /* pause  api operation */
    {    { reject_opr,  W_IDLE,     SUBIDLE },
         { reject_opr,  W_IDLE,     SUBCOLD },
         { reject_opr,  W_IDLE,     SUBWARM },
         { request_opr, W_RUNNING,  SUBSTOPPING },
         { request_opr, W_RUNNING,  SUBSTOPPING },
         { reject_opr,  W_RUNNING,  SUBSTOPPING }
    },
    RPAUSE		            /* RPAUSE request parameter */
  },
  { WDX_GROUP_EXCEPTION,		    /* exception api operation */
    {    { reject_opr, W_IDLE,     SUBIDLE },
         { reject_opr, W_IDLE,     SUBCOLD },
         { reject_opr, W_IDLE,     SUBWARM },
         { reject_opr, W_RUNNING,  SUBSILENCE },
         { action_opr, W_RUNNING,  SUBRUNNING },
         { reject_opr, W_RUNNING,  SUBSTOPPING }
    },
    ACSHUT		            /* ACSHUT action parameter */
  },
  { WDX_MEMBER_UP,		    /* member up api operation */
    /* allowed in every substatus */
    {    { action_opr, W_IDLE,     SUBCOLD },
         { aldone_opr, W_IDLE,     SUBCOLD },
         { aldone_opr, W_IDLE,     SUBWARM },
         { aldone_opr, W_RUNNING,  SUBSILENCE },
         { aldone_opr, W_RUNNING,  SUBRUNNING },
         { aldone_opr, W_RUNNING,  SUBSTOPPING }
    },
    ACUP			    /* ACUP action parameter */
  },
  { WDX_PROCESS_KILL,		    /* process kill api operation */
    /* allowed in every substatus */
    {    { action_opr, W_IDLE,     SUBIDLE },
         { action_opr, W_IDLE,     SUBCOLD },
         { action_opr, W_IDLE,     SUBWARM },
         { action_opr, W_RUNNING,  SUBSILENCE },
         { action_opr, W_RUNNING,  SUBRUNNING },
         { action_opr, W_RUNNING,  SUBSTOPPING }
    },
    ACLOCKILL			    /* ACLOCKILL parameter required */
  }, 
  { WDX_WAIT_FOR_RESERVE, 
    {    { reject_opr, W_IDLE,     SUBIDLE },
         { reject_opr, W_IDLE,     SUBCOLD },
         { reject_opr, W_IDLE,     SUBWARM },
         { wait_opr,   W_RUNNING,  SUBSILENCE },
         { wait_opr,   W_RUNNING,  SUBRUNNING },
         { reject_opr, W_RUNNING,  SUBSTOPPING }
    },
    WAIT_FOR_RESERVE		    /* wait_for_reserve parameter */
  },
  { WDX_WAIT_FOR_MOUNT,   
    {    { reject_opr, W_IDLE,     SUBIDLE },
         { reject_opr, W_IDLE,     SUBCOLD },
         { reject_opr, W_IDLE,     SUBWARM },
         { wait_opr,   W_RUNNING,  SUBSILENCE },
         { wait_opr,   W_RUNNING,  SUBRUNNING },
         { reject_opr, W_RUNNING,  SUBSTOPPING }
    },
    WAIT_FOR_MOUNT		    /* wait_for_mount parameter */
  },
  { WDX_LOG,            
    {    { local_opr, W_IDLE,     SUBIDLE },
         { local_opr, W_IDLE,     SUBCOLD },
         { local_opr, W_IDLE,     SUBWARM },
         { local_opr, W_RUNNING,  SUBSILENCE },
         { local_opr, W_RUNNING,  SUBRUNNING },
         { local_opr, W_RUNNING,  SUBSTOPPING }
    },
    WDX_LOG			    /* local_opr parameter */
  },
  { WDX_CONSTANT,       
    {    { local_opr, W_IDLE,     SUBIDLE },
         { local_opr, W_IDLE,     SUBCOLD },
         { local_opr, W_IDLE,     SUBWARM },
         { local_opr, W_RUNNING,  SUBSILENCE },
         { local_opr, W_RUNNING,  SUBRUNNING },
         { local_opr, W_RUNNING,  SUBSTOPPING }
    },
    WDX_CONSTANT 
  },
  { WDX_SCRIPT,       
    {    { local_opr, W_IDLE,     SUBIDLE },
         { local_opr, W_IDLE,     SUBCOLD },
         { local_opr, W_IDLE,     SUBWARM },
         { local_opr, W_RUNNING,  SUBSILENCE },
         { local_opr, W_RUNNING,  SUBRUNNING },
         { local_opr, W_RUNNING,  SUBSTOPPING }
    },
    WDX_SCRIPT
  },
  { WDX_LIST,           
    {    { local_opr, W_IDLE,     SUBIDLE },
         { local_opr, W_IDLE,     SUBCOLD },
         { local_opr, W_IDLE,     SUBWARM },
         { local_opr, W_RUNNING,  SUBSILENCE },
         { local_opr, W_RUNNING,  SUBRUNNING },
         { local_opr, W_RUNNING,  SUBSTOPPING }
    },
    WDX_LIST 
  },
  { WDX_MASTERNAME,     
    {    { local_opr, W_IDLE,     SUBIDLE },
         { local_opr, W_IDLE,     SUBCOLD },
         { local_opr, W_IDLE,     SUBWARM },
         { local_opr, W_RUNNING,  SUBSILENCE },
         { local_opr, W_RUNNING,  SUBRUNNING },
         { local_opr, W_RUNNING,  SUBSTOPPING }
    },
    WDX_MASTERNAME 
  },
  { WDX_LOCALNAME,      
    {    { local_opr, W_IDLE,     SUBIDLE },
         { local_opr, W_IDLE,     SUBCOLD },
         { local_opr, W_IDLE,     SUBWARM },
         { local_opr, W_RUNNING,  SUBSILENCE },
         { local_opr, W_RUNNING,  SUBRUNNING },
         { local_opr, W_RUNNING,  SUBSTOPPING }
    },
    WDX_LOCALNAME 
  },
  { WDX_STATUS,         
    {    { local_opr, W_IDLE,     SUBIDLE },
         { local_opr, W_IDLE,     SUBCOLD },
         { local_opr, W_IDLE,     SUBWARM },
         { local_opr, W_RUNNING,  SUBSILENCE },
         { local_opr, W_RUNNING,  SUBRUNNING },
         { local_opr, W_RUNNING,  SUBSTOPPING }
    },
    WDX_STATUS
  },
  { WDX_DECLARE,        
    {    { reject_opr, W_IDLE,     SUBIDLE },
         { reject_opr, W_IDLE,     SUBCOLD },
         { reject_opr, W_IDLE,     SUBWARM },
         { delay_opr,  W_RUNNING,  SUBSILENCE },
         { lock_opr,   W_RUNNING,  SUBRUNNING },
         { reject_opr, W_RUNNING,  SUBSTOPPING }
    },
    DECLARING 
  },
  { WDX_ERASE,          
    {    { reject_opr, W_IDLE,     SUBIDLE },
         { reject_opr, W_IDLE,     SUBCOLD },
         { reject_opr, W_IDLE,     SUBWARM },
         { delay_opr,  W_RUNNING,  SUBSILENCE },
         { lock_opr,   W_RUNNING,  SUBRUNNING },
         { reject_opr, W_RUNNING,  SUBSTOPPING }
    },
    ERASING  
  },	
  { WDX_RESERVE,        
    {    { reject_opr, W_IDLE,     SUBIDLE },
         { reject_opr, W_IDLE,     SUBCOLD },
         { reject_opr, W_IDLE,     SUBWARM },
         { delay_opr,  W_RUNNING,  SUBSILENCE },
         { lock_opr,   W_RUNNING,  SUBRUNNING },
         { reject_opr, W_RUNNING,  SUBSTOPPING }
    },
    RESERVING 
  },
  { WDX_MOUNT,          
    {    { reject_opr, W_IDLE,     SUBIDLE },
         { reject_opr, W_IDLE,     SUBCOLD },
         { reject_opr, W_IDLE,     SUBWARM },
         { delay_opr,  W_RUNNING,  SUBSILENCE },
         { lock_opr,   W_RUNNING,  SUBRUNNING },
         { reject_opr, W_RUNNING,  SUBSTOPPING }
    },
    MOUNTING  
  },
  { WDX_FREE,           
    {    { reject_opr, W_IDLE,     SUBIDLE },
         { reject_opr, W_IDLE,     SUBCOLD },
         { reject_opr, W_IDLE,     SUBWARM },
         { delay_opr,  W_RUNNING,  SUBSILENCE },
         { lock_opr,   W_RUNNING,  SUBRUNNING },
         { reject_opr, W_RUNNING,  SUBSTOPPING }
    },
    FREEING   
  },
  { WDX_FAULT,          
    {    { reject_opr, W_IDLE,     SUBIDLE },
         { reject_opr, W_IDLE,     SUBCOLD },
         { reject_opr, W_IDLE,     SUBWARM },
         { delay_opr,  W_RUNNING,  SUBSILENCE },
         { lock_opr,   W_RUNNING,  SUBRUNNING },
         { reject_opr, W_RUNNING,  SUBSTOPPING }
    },
    FAULTING  
  },
  { WDX_GETMASTER,		    /* getmaster api operation */
    {    { reject_opr, W_IDLE,     SUBIDLE },
         { reject_opr, W_IDLE,     SUBCOLD },
         { reject_opr, W_IDLE,     SUBWARM },
         { masdel_opr,  W_RUNNING,  SUBSILENCE },
         { master_opr, W_RUNNING,  SUBRUNNING },
         { reject_opr, W_RUNNING,  SUBSTOPPING }
    },
    GETTING			    /* parameter */
  },
  { WDX_GET,			    /* get api operation */
    {    { reject_opr, W_IDLE,     SUBIDLE },
         { reject_opr, W_IDLE,     SUBCOLD },
         { reject_opr, W_IDLE,     SUBWARM },
         { delay_opr,  W_RUNNING,  SUBSILENCE },
         { lock_opr,   W_RUNNING,  SUBRUNNING },
         { reject_opr, W_RUNNING,  SUBSTOPPING }
    },
    GETTING			    /* parameter */
  },
  { WDX_MASTER,			    /* master api operation */
    {    { masidl_opr, W_IDLE,     SUBCOLD },
         { reject_opr, W_IDLE,     SUBCOLD },
         { reject_opr, W_IDLE,     SUBWARM },
         { masdel_opr,  W_RUNNING,  SUBSILENCE },
         { master_opr, W_RUNNING,  SUBRUNNING },
         { reject_opr, W_RUNNING,  SUBSTOPPING }
    },
    RESERVING			    /* parameter */
  },
  { WDX_NOMASTER,		    /* nomaster api operation */
    {    { reject_opr, W_IDLE,     SUBIDLE },
         { reject_opr, W_IDLE,     SUBCOLD },
         { reject_opr, W_IDLE,     SUBWARM },
         { masdel_opr, W_RUNNING,  SUBSILENCE },
         { master_opr, W_RUNNING,  SUBRUNNING },
         { reject_opr, W_RUNNING,  SUBSTOPPING }
    },
    FREEING			    /* parameter */
  },
  { 0, 
    {    { NULL, 0, 0 },
         { NULL, 0, 0 },
         { NULL, 0, 0 },
         { NULL, 0, 0 },
         { NULL, 0, 0 },
         { NULL, 0, 0 }
    },
    0
  }
};


struct event_table reqevt[] =
{
 { RRUN,		            /* RRUN request event */
    {    { accept_act,   W_IDLE,     SUBIDLE },
         { accept_act,   W_IDLE,     SUBCOLD },
         { conflict_act, W_IDLE,     SUBIDLE },
         { accept_act,   W_RUNNING,  SUBSILENCE },
         { accept_act,   W_RUNNING,  SUBRUNNING },
         { conflict_act, W_RUNNING,  SUBRUNNING }
    },
    RRUN 			    /* parameter */
  },
  { RSTOP,		            /* RSTOP request event */
    {    { accept_act,   W_IDLE,     SUBIDLE },
         { conflict_act, W_IDLE,     SUBIDLE },
         { conflict_act, W_IDLE,     SUBIDLE },
         { accept_act,   W_RUNNING,  SUBSILENCE },
         { accept_act,   W_RUNNING,  SUBRUNNING },
         { conflict_act, W_RUNNING,  SUBRUNNING }
    },
    RSTOP 			    /* parameter */
  },
  { RSTART,		            /* RRUN request event */
    {    { accept_act,   W_IDLE,     SUBIDLE },
         { conflict_act, W_IDLE,     SUBIDLE },
         { conflict_act, W_IDLE,     SUBIDLE },
         { reject_act,   W_RUNNING,  SUBSILENCE },
         { reject_act,   W_RUNNING,  SUBRUNNING },
         { reject_act,   W_RUNNING,  SUBSTOPPING }
    },
    RSTART 			    /* parameter */
  },
  { RPAUSE,		            /* RPAUSE request event */
    {    { accept_act,   W_IDLE,     SUBIDLE },
         { conflict_act, W_IDLE,     SUBIDLE },
         { conflict_act, W_IDLE,     SUBIDLE },
         { accept_act,   W_RUNNING,  SUBSILENCE },
         { accept_act,   W_RUNNING,  SUBRUNNING },
         { conflict_act, W_RUNNING,  SUBRUNNING }
    },
    RPAUSE 			    /* parameter */
  },
  { RPAUSESTART,		            /* RPAUSESTART request event */
    {    { accept_act,   W_IDLE,     SUBIDLE },
         { conflict_act, W_IDLE,     SUBIDLE },
         { conflict_act, W_IDLE,     SUBIDLE },
         { accept_act,   W_RUNNING,  SUBSILENCE },
         { accept_act,   W_RUNNING,  SUBRUNNING },
         { conflict_act, W_RUNNING,  SUBRUNNING }
    },
    RPAUSESTART 			    /* parameter */
  },
  { 0, 
    {    { NULL, 0, 0 },
         { NULL, 0, 0 },
         { NULL, 0, 0 },
         { NULL, 0, 0 },
         { NULL, 0, 0 },
         { NULL, 0, 0 }
    },
    0
  }
};

struct event_table ansevt[] =
{
  { AOK,				    /* AOK answer event */
    {    { exclude_act, W_IDLE,     SUBIDLE },
         { exclude_act, W_IDLE,     SUBCOLD },
         { exclude_act, W_IDLE,     SUBWARM },
         { exclude_act, W_RUNNING,  SUBSILENCE },
         { exclude_act, W_RUNNING,  SUBRUNNING },
         { exclude_act, W_RUNNING,  SUBSTOPPING }
    },
    AOK				    /* parameter */
  },
  { ANONOK,		            /* ANONOK answer event */
    {    { exclude_act, W_IDLE,     SUBIDLE },
         { exclude_act, W_IDLE,     SUBIDLE },
         { exclude_act, W_IDLE,     SUBIDLE },
         { exclude_act, W_RUNNING,  SUBSILENCE },
         { exclude_act, W_RUNNING,  SUBRUNNING },
         { exclude_act, W_RUNNING,  SUBRUNNING }
    },
    ANONOK			    /* parameter */
  },
  { 0, 
    {    { NULL, 0, 0 },
         { NULL, 0, 0 },
         { NULL, 0, 0 },
         { NULL, 0, 0 },
         { NULL, 0, 0 },
         { NULL, 0, 0 }
    },
    0
  }
};

struct event_table actevt[] =
{
  { ACRUN,		            /* CRUN action event */
    {    { process_act, W_RUNNING, SUBSILENCE },
         { process_act, W_RUNNING, SUBSILENCE },
         { ignore_act,  W_IDLE,    SUBWARM },
         { ignore_act,  W_RUNNING, SUBSILENCE },
         { ignore_act,  W_RUNNING, SUBRUNNING },
         { ignore_act,  W_RUNNING, SUBSTOPPING }
    },
    ACRUN 			    /* parameter */
  },
  { ACSTOP,		            /* CSTOP action event */
    {    { ignore_act,  W_IDLE,    SUBIDLE },
         { process_act, W_IDLE,    SUBIDLE },
         { process_act, W_IDLE,    SUBIDLE },
         { process_act, W_IDLE,    SUBIDLE },
         { process_act, W_IDLE,    SUBIDLE },
         { process_act, W_IDLE,    SUBIDLE }
    },
    ACSTOP			            /* parameter */
  },
  { ACSTART,		            /* CSTART action event */
    {    { process_act,W_RUNNING,  SUBSILENCE },
         { ignore_act, W_IDLE,     SUBCOLD },
         { process_act,W_RUNNING,  SUBSILENCE },
         { ignore_act, W_RUNNING,  SUBSILENCE },
         { ignore_act, W_RUNNING,  SUBRUNNING },
         { ignore_act, W_RUNNING,  SUBSTOPPING }
    },
    ACSTART			    /* parameter */
  },
  { ACPAUSESTART,		            /* CPAUSESTART action event */
    {    { process_act,W_RUNNING,  SUBSILENCE },
         { ignore_act, W_IDLE,     SUBCOLD },
         { process_act,W_RUNNING,  SUBSILENCE },
         { ignore_act, W_RUNNING,  SUBSILENCE },
         { ignore_act, W_RUNNING,  SUBRUNNING },
         { ignore_act, W_RUNNING,  SUBSTOPPING }
    },
    ACPAUSESTART			    /* parameter */
  },
  { ACPAUSE,		            /* CSTOP action event */
    {    { ignore_act,  W_IDLE,    SUBIDLE },
         { process_act, W_IDLE,    SUBIDLE },
         { process_act, W_IDLE,    SUBIDLE },
         { process_act, W_IDLE,    SUBIDLE },
         { process_act, W_IDLE,    SUBIDLE },
         { process_act, W_IDLE,    SUBIDLE }
    },
    ACPAUSE			            /* parameter */
  },
  { ACPAUSETIMER,		            /* PAUSE action event */
    {    { process_act,  W_IDLE,    SUBIDLE },
         { ignore_act, W_IDLE,    SUBIDLE },
         { ignore_act, W_IDLE,    SUBIDLE },
         { ignore_act, W_RUNNING,  SUBSILENCE },   /* 6-Feb-1996 */
         { ignore_act, W_RUNNING,  SUBRUNNING },   /*            */
         { ignore_act, W_IDLE,    SUBIDLE },
    },
    ACPAUSETIMER                            /* parameter */
  },
  { ACKILL,		            /* ACKILL action event */
    {    { process_act,   W_FAULT,     SUBFAULT },
         { process_act,   W_FAULT,     SUBFAULT },
         { process_act,   W_FAULT,     SUBFAULT },
         { process_act,   W_FAULT,     SUBFAULT },
         { process_act,   W_FAULT,     SUBFAULT },
         { process_act,   W_FAULT,     SUBFAULT },
    },
    ACKILL		            /* parameter */
  },
  { ACSHUT,		            /* ACSHUT action event */
    {    { ignore_act, W_IDLE,     SUBIDLE },
         { ignore_act, W_IDLE,     SUBCOLD },
         { ignore_act, W_IDLE,     SUBWARM },
         { process_act,W_RUNNING,  SUBSILENCE },
         { process_act,W_RUNNING,  SUBRUNNING },
         { process_act,W_RUNNING,  SUBSTOPPING },
    },
    ACSHUT			    /* OPERATOR parameter */
  },
  { ACLOCKILL,		            /* ACLOCKILL action event */
    {    { process_act,   W_FAULT,     SUBFAULT },
         { process_act,   W_FAULT,     SUBFAULT },
         { process_act,   W_FAULT,     SUBFAULT },
         { process_act,   W_FAULT,     SUBFAULT },
         { process_act,   W_FAULT,     SUBFAULT },
         { process_act,   W_FAULT,     SUBFAULT }
    },
    ACLOCKILL			    /* parameter */
  },
  { ACUP,		            /* ACUP action event */
    {    { process_act, W_RUNNING, SUBSILENCE },
         { process_act, W_RUNNING, SUBSILENCE },
         { ignore_act,  W_IDLE,    SUBWARM },
         { ignore_act,  W_RUNNING, SUBSILENCE },
         { ignore_act,  W_RUNNING, SUBRUNNING },
         { ignore_act,  W_RUNNING, SUBSTOPPING }
    },
    ACUP 			    /* parameter */
  },
  { ACTIMER,		            /* ACTIMER action event */
    {    { ignore_act, W_IDLE,     SUBIDLE },
         { ignore_act, W_IDLE,     SUBCOLD },
         { ignore_act, W_IDLE,     SUBWARM },
	 { process_act,W_RUNNING,  SUBRUNNING },
         { ignore_act, W_RUNNING,  SUBRUNNING },
         { ignore_act, W_RUNNING,  SUBSTOPPING }
    },
    ACTIMER			    /* parameter */
  },
  { 0, 
    {    { NULL, 0, 0 },
         { NULL, 0, 0 },
         { NULL, 0, 0 },
         { NULL, 0, 0 },
         { NULL, 0, 0 },
         { NULL, 0, 0 }
    },
    0
  }
};

#endif /* WDX_KERNEL_MODULE */

#endif /* WDX_KERNEL_H */
