/*
 *   WDX: WitchDoctor-WDX, versatile distributed lock manager
 *   Copyright (C) 2003 Dominique Chabord,
 *   Please refer to credits section below
 *
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   ************************************************
 *
 *   History and credits:
 *   First version of WitchDoctor-WDX was based on:
 *
 *   DECwatchdog/Autopilot, watchdog and lock manager
 *   Copyright (C) 2003 HP
 *
 *   Developed by
 *     Computer Special Systems, Annecy France
 *     A Division of Digital Equipment Corporation
 *   and
 *     Custom-Systems, Annecy France
 *     An engineering center of Compaq
 *
 *   Provided under GNU-GPL terms by HP, 30 June 2003
 *   GNU-GPL replaces previous license statements
 *
 *   Rebranded, ported and modified for Shaman-X project, 15 July 2003
 *   by Dominique Chabord
 *   and maintained by other contributors as mentionned in the code
 *
 *   ************************************************
 *
 *   WDX is promoted by Shaman-X project
 *
 *   Check for new versions and documentation at
 *   http://www.shaman-x.org
 *
 *   ************************************************
 *
 *   Most of WDX code is designed to run unchanged on
 *   different operating systems, including at V1.0 time:
 *   Gnu-Linux, WindowsNT, DigitalUNIX and OpenVMS.
 *   Operating system specifics are identified by dedicated files.
 *
 *   Evolutions
 *   ----------
 *   July-2003     Dominique Chabord    WitchDoctor-WDX V1.0
 *                 Creation
 *
 *   ************************************************
 *
 *
 *	wdx_specific_gnux.c
 *
 *	Module including GNU-Linux specific routines used by WDX
 *	Public routines follow the naming convention :
 *		spc_xxxxxxx()
 *
 *	Miscellaneous routines
 *		int spc_unlink()
 *		int spc_strcasecmp()
 *
 *	Routines to check license and release license units
 *              int  spc_probe_license()
 *              void spc_release_license()
 *
 *	Routines to check mount point directory
 *              int  spc_check_mount_point()
 *
 *      Routines to manage timers
 *
 *	Routines to manage process
 *	        int  spc_create_daemon()
 *	        void spc_trigger_script()
 *
 ************************************************************************/
/*
 *
 *  21-Apr-94 -- Cornu
 *      Creation
 *
 * MK 0001  M. Kennealy  Added debug routine to output timestamp
 *
 * CDM 001  C. de Mare . No license required now.
 */

#undef WDX_SPECIFIC__MODULE
#define WDX_SPECIFIC_MODULE

#include	"wdx.h"
#include	"wdx_config.h"
#include <signal.h>
#include <unistd.h>

/* add LIONEL TRICON */
#include <sys/time.h>
struct timeval tv;
struct timezone tz;
/* fin add LIONEL TRICON */
/*
 * Data declaration
 */
/*static int retsignal;*/
int retsignal;
int sleep_finished_flag = 0;

/* LOCAL ROUTINES prototypes */
int set_fifo_nonblocking(int d);
int set_fifo_blocking(int d);

void sleep_timer_exp()
{
  sleep_finished_flag = 1;
}

/* char *spc_get_process_name()
 * Return the real process name expurged from path "/"
 *
 * Argument : pt to process name
 */
char *spc_get_process_name(char *val)
{
char *p1;
char *p2;

   p1 = val;
   while( (p2 = strstr(p1,"/")) != NULL )
		  p1 = ++p2;
   return(p1);
}

/* int spc_get_running_mission()
 * Return a list of all the mission currently running
 *
 * Argument : None
 *
 * Return   : NULL if error else pt to mission_list queue
 */
struct mission_list *spc_get_running_mission()
{
DIR                      *dirp; /* pt to DIR */
struct DIRENTRY	     *direntry; /* pt to directory entry */
char                       *pt; /* extension pointer */
char      mission[MISSION_LEN]; /* mission name */
struct mission_list      *head; /* pt on queue head */
struct mission_list   *current; /* pt on current queue element */
struct mission_list       *new; /* pt on new queue element */

	head = NULL;
	if ( (dirp = opendir(wdxdirectory)) == NULL )
	{
		 sprintf(messagestring,"Can't open %s directory",wdxdirectory);
		 PERROR_B
		 messagestring
		 PERROR_E;
		 return(head);
	}
	while( (direntry = readdir(dirp)) )
	{
		if ( (pt = strstr(direntry->d_name,DATA_EXTENSION)) != NULL )
		{
			 /* attach and test sharable memory segment */
			 strncpy(mission,direntry->d_name,MISSION_LEN);
			 mission[pt-direntry->d_name] = '\0';
			 if ( strlen(direntry->d_name) ==
				 (strlen(mission) + strlen(DATA_EXTENSION)) )
			 {
		/* add to queue list */
		new = (struct mission_list *)
						  malloc(sizeof(struct mission_list));
			if ( new == NULL )
		{
			current = head;
			while(current)
			{
			head = current->next;
			free(current);
			current = head;
			}
			PERROR_B
			"Memory allocation for mission_list"
			PERROR_E;
			return(NULL);
		}
			/* link to queue */
		new->next = NULL;
		strncpy(new->mission,mission,MISSION_LEN);
		if ( head == NULL )
					 head = new;
				else current->next = new;
		current = new;
			 }
		}
	}
	closedir(dirp);
	return(head);
}

/* int spc_path_init()
 * Set argument to wdx path
 *
 * Argument : pt to global path
 *
 * return : 0 OK / -1 error
 *
 */
int spc_path_init(p)
char **p;
{
FILE *f;

	if (strlen(wdx_path1)+strlen(pn)+strlen(wdx_path2) > MAXPROGPATHLEN)
		return (-1);
	strcpy (wdx_path, wdx_path1);
	strcat (wdx_path, pn);
	strcat (wdx_path, wdx_path2);

	*p = wdx_path;

   if ( (f = fopen(*p,"r")) == NULL )
		return(-1);
   fclose(f);
   return(0);
}
/* void spc_rename()
 * Rename a file name.ext to name.ext.0
 *
 * Argument : pt to file name
 *
 * return : None
 *
 */
void spc_rename(log)
char *log;
{
char logrename[STRING_LEN];

   sprintf(logrename,LOG0_FILE,log);
   rename(log,logrename);
}
/* int spc_unlink()
 * Delete a file
 *
 * Argument : pt file
 *
 * Return :  0 OK
 *          -1 error
 */
int spc_unlink(f)
char *f;
{
   return(unlink(f));
}

/* int spc_priority_funct()
 * Specific priority_funct routine.
 * Check the ASCII value passed as argument is an integer value between -20 and
 * +20
 * In case of success set the field priority of the support_file structure
 * In case of error set messagestring
 *
 * Argument : pt to support_file
 *	      pt to ASCII value
 *
 * Return :  0 .. OK
 *          -1 .. error messagestring is set
 */
int spc_priority_funct(c,w)
int *c;
char *w;
{
int _priovalue;
char      *_p;
char      *_v;

   _p = w;
   sprintf(messagestring,
		   "Priority value (%s) must be between -20 and +20",
		   _p);
   if ( *_p == '-' )
	_v = _p + 1;
   else _v = _p;
   if ( lib_is_numeric(_v) )
   {
		return(-1);
   }
   _priovalue = atoi(_p);
   if ( _priovalue < -20 )
   {
		return(-1);
   }
   if ( _priovalue > 20 )
   {
		return(-1);
   }
   *c = _priovalue;
   return(0);
}

/* int spc_strcasecmp
 * Case insentive string comparison
 *
 * Argument : pt to string 1
 *	      pt to string 2
 *
 * Return   : >0 if s1 lexicographically greater than s2
 *            =0 if s1 lexicographically equal to s2
 *            <0 if s1 lexicographically less than s2
 */
int spc_strcasecmp(s1,s2)
char *s1;
char *s2;
{
	return(strcasecmp(s1,s2));
}

/* int spc_check_mount_point()
 *
 */
/* modif LIONEL TRICON */
int spc_check_mount_point(mpoint,md)
char                   *mpoint;   /* mount point to check */
enum wdx_modes              md;   /* WDX_PROCESS or WDX_APPLICATION mode */
{
int n, i, ret, found;
		char foo[2048];
		ret = 0;
		found = 0;
		/* now look for all local mounted disks */
/* pas propre -> a modifier ...\n"); */
		sprintf(foo,"(mount -t nonfs | grep \"%s \") > /dev/null", mpoint);
		ret = system(foo);
		if (ret==0) {
		  found = 1;
		  } else {
			if (!found)
			{
				 ret = -1;
				 sprintf(messagestring,
					"%s not a local mount point directory",mpoint);
				 if ( md == WDX_PROCESS )
				 {
					  PWARNING_B
					  messagestring
					  PWARNING_E;
				 }
			}
		  }
		return(ret);

}
/* fin modif LIONEL TRICON */

void catch_signal(sig)
int sig;
{
	retsignal = sig;
}

#ifndef WDX_BUILD_API
/* int spc_create_daemon()
 * Start the daemon process. Only one daemon can be started at one time.
 *
 * Argument : pt to mission name
 *	      pt to group data structure
 *	      pt to scheduler function
 *
 * Return :  0 .. daemon started
 *          -1 .. daemon exits
 */
int spc_create_daemon(mis,gp,mess,scheduler)
char *mis;		 /* mission name */
struct group *gp;	 /* group data base */
struct message *mess; /* pt to message structure */
int (* scheduler)();	 /* scheduler function pt */
{
int fd;      /* file descriptor */
int error;   /* exit on error */
struct function_table *tpt;
struct sigvec vec1;
struct sigvec vec2;
char *p;
int crit;

#ifdef OpenBSD
int status;
#endif

	/* start as a daemon */
#ifndef DAEMON_DEBUG
	switch (fork())
		{
	case -1:        /* error */
				PERROR_B
		"Can't create child process %m"
				PERROR_E;
		return(-1);
				break;
	case  0:       /* child process */
		break;
	default:       /* parent process : return OK */
			vec1.sv_handler  = catch_signal;
		vec1.sv_mask = 0;
		vec1.sv_flags = SV_INTERRUPT;
		if ( sigvec(SIGUSR1,&vec1,(struct sigvec *)0) == -1 )
		{
			PERROR_B
			"sigvec error %m"
			PERROR_E;
		}
			vec2.sv_handler  = catch_signal;
		vec2.sv_mask = 0;
		vec2.sv_flags = SV_INTERRUPT;
		if ( sigvec(SIGUSR2,&vec2,(struct sigvec *)0) == -1 )
		{
			PERROR_B
			"sigvec error %m"
			PERROR_E;
		}

/* modified DM 16/10/03 */
#ifdef OpenBSD
		wait (&status);
#else
		wait((union wait *)0);
#endif

		if ( retsignal == SIGUSR1 )
			return(0);
		else return(1);
				break;
	   }
#endif /*DAEMON_DEBUG*/

	/* child process - define a correct daemon process environment	*/

	error = lib_process_init(COLD_INIT_PROCESS,mis,gp,NULL,mess);
	if ( error == 0 )
	{
		 openlog(pn,LOG_PID,LOG_DAEMON);    /* activate syslog */
		 if ( gp->memberlist[localindex].type == MASTER_SUBSTITUTE_T )
		  p = "Process is configured for MASTER-SUBSTITUTE functions";
		 else p = "Process is configured for SATELLITE functions only";
	 PCOMPLEMENT_B
	 p
	 PCOMPLEMENT_E;
	 sprintf(messagestring,
		 "Mission \"%s\" started. %s",gp->mission,p);
	 syslog(LOG_INFO,messagestring);
		 /* redirect messages to log file */
		 lib_redirect_output(gp->log,gp->logfile);
		 sprintf(messagestring,"Wdx started");
		 PINFOR_B
		 messagestring
		 PINFOR_E;

		 /* Do not close all open file descriptors, but mark them	    */
		 /* with close-on-exec flags	if descriptor is higher than stderr */
		 for (fd=getdtablesize(); fd > 2; fd--)
		 fcntl(fd,F_SETFD,FD_CLOEXEC);

#ifndef DAEMON_DEBUG
		 umask(022);			/* no secret data --> 022	*/
		 chdir("/");			/* working dir changed to /	*/

#ifdef OpenBSD
		setpgrp(0,0);			/* OpenBSD requires the 2 parameters */
#else
		setpgrp();				/* Linux doesn't */
#endif

/*         setpgrp(0,getpid());*/	/* def. process group leader 	*/

		 /* make immune to certain IO signals */
		 signal(SIGHUP,SIG_IGN);
		 signal(SIGINT,SIG_IGN);
		 signal(SIGQUIT,SIG_IGN);
		 signal(SIGPIPE,SIG_IGN);
		 signal(SIGTERM,SIG_IGN);
		 signal(SIGTSTP,SIG_IGN);
		 signal(SIGTTIN,SIG_IGN);
		 signal(SIGTTOU,SIG_IGN);
		 signal(SIGXCPU,SIG_IGN);
		 signal(SIGXFSZ,SIG_IGN);
		 signal(SIGVTALRM,SIG_IGN);
		 signal(SIGPROF,SIG_IGN);
/* delete LIONEL TRICON */
		 /* signal(SIGLOST,SIG_IGN); */
/* fin delete LIONEL TRICON */
		 signal(SIGUSR1,SIG_IGN);
		 signal(SIGUSR2,SIG_IGN);

		 /* disassociate from terminal */
		 if ((fd=open("/dev/tty",O_RDWR)) >= 0)
		 {
			  if (ioctl(fd,TIOCNOTTY,(char *)0))
			  {
				   error++;
				   PERROR_B
				   "Can't drop control terminal %m"
				   PERROR_E;
			  }
		 close(fd);
		 }

		 /* redirect i/o */
		 if (freopen("/dev/null","r",stdin)==((FILE *)0))
		 {
			  error++;
			  PERROR_B
			  "Can't redirect stdin to /dev/null %m"
			  PERROR_E;
		 }
		 if (freopen("/dev/null","w",stdout)==((FILE *)0))
		 {
			  error++;
			  PERROR_B
			  "Can't redirect stdout to /dev/null %m"
			  PERROR_E;
		 }

		 /* lock text/data in memory	*/
/* delete LIONEL TRICON */
/* on peut utiliser mlockall() a la place ?? */
		 /*if ( plock(PROCLOCK) == -1 )
		 {
			 error++;
			 PERROR_B
			 "Can't lock text and data segments %m"
			 PERROR_E;
		 }*/
/* fin delete LIONEL TRICON */

		 /* set process priority */
		 if (setpriority(PRIO_PROCESS,getpid(),gp->priority))
		 {
			 error++;
			 PERROR_B
			 "Can't set process priority %m"
			 PERROR_E;
		 }

#endif /*DAEMON_DEBUG*/
	}

	if ( error )
	{
#ifndef DAEMON_DEBUG
	kill(getppid(),SIGUSR2);    /* notify parent process */
#endif /*DAEMON_DEBUG*/
	error = -1;
	}
	if(!error)
	{
#ifndef DAEMON_DEBUG
	kill(getppid(),SIGUSR1);    /* notify parent process */
#endif /*DAEMON_DEBUG*/
		error = (* scheduler)();    /* loop returns on fire only */
	}

	if ( error != 1 )
	{
		 sprintf(messagestring,
				 "Fatal error in \"%s\" mission.(refer to %s file)",
				 gp->mission,gp->logfile);
		 crit = LOG_CRIT;
	}
	else
	{
		 sprintf(messagestring,
				 "Mission \"%s\" terminated",
				 gp->mission);
		  crit = LOG_INFO;
	}
	syslog(crit,messagestring);
	return (-1);
}
#endif /* WDX_BUILD_API */

/* void spc_trigger_script()
 * Trigger a script p and send an associated user message m
 *
 * Argument : pt to script name
 *	      pt to information message
 */
void spc_trigger_script(p,m)
char *p;
char *m;
{
char buffer[STRING_LEN];

   if(m)
   {
	  PINFOR_B
	  m
	  PINFOR_E;
   }
   if(p)
   {
	  sprintf(buffer,"%s &",p);
	  system(buffer);
   }
}

/* int spc_open_socket()
 * Open a socket
 *
 * Argument : pt to socket descriptor to fill in
 *            address format
 *	      socket type
 *	      protocol
 *
 * Return   : 0 if OK / -1 if error
 */
int spc_open_socket(WDX_SOCKET *s,int af,int type,int proto)
{
int ret;

	ret = socket(af,type,proto);
	if (ret == -1)
	return(-1);
	*s = ret;
	return(0);
}

/* int spc_close_socket()
 * Close a socket
 *
 * Argument : socket descriptor to close
 *
 * Return   : 0 if OK / -1 if error
 */
int spc_close_socket(WDX_SOCKET s)
{
	return(close(s));
}

/* int spc_bind_socket()
 * Bind a name to a socket
 *
 * Argument : socket descriptor
 *            name pointer
 *	      name length
 *
 * Return   : 0 if OK / -1 if error
 */
int spc_bind_socket(WDX_SOCKET s,struct sockaddr *name,int namelen)
{
int ret;

	ret = bind(s,name,namelen);
	if ( ret == -1 )
		 return(-1);
	return(0);
}

/* int spc_ioctl_socket()
 * Set or get socket information
 *
 * Argument : socket descriptor
 *            name pointer
 *	      name length
 *
 * Return   : 0 if OK / -1 if error
 */
int spc_ioctl_socket(WDX_SOCKET s,struct sockaddr *name,int namelen)
{
int ret;

	ret = bind(s,name,namelen);
	if ( ret == -1 )
		 return(-1);
	return(0);
}

/* int spc_setopt_socket()
 * Set option to a socket
 *
 * Argument : socket descriptor
 *            level
 *	      option name
 *            option value
 *            option length
 *
 * Return   : 0 if OK / -1 if error
 */
int spc_setopt_socket(WDX_SOCKET s,int level,int optname,char *optval,int optlen)
{
int ret;

	ret = setsockopt(s,level,optname,optval,optlen);
	if ( ret == -1 )
		 return(-1);
	return(0);
}

/* int spc_getopt_socket()
 * Get option of a socket
 *
 * Argument : socket descriptor
 *            level
 *	      option name
 *            option value
 *            pt option length
 *
 * Return   : 0 if OK / -1 if error
 */
int spc_getopt_socket(WDX_SOCKET s,int level,int optname, char *optval,int *optlen)
{
int ret;

	ret = getsockopt(s,level,optname,optval,optlen);
	if ( ret == -1 )
		 return(-1);
	return(0);
}

/* int spc_write_socket()
 * Send a message through a socket
 *
 * Argument : socket descriptor
 *            message pt
 *	      message length
 *            flags
 *            destination pt
 *	      destination length
 *
 * Return   : 0 if OK / -1 if error
 */
int spc_write_socket(WDX_SOCKET s,char *b,int bl,int f,struct sockaddr *a,int al)
{
	if (  sendto(s,b,bl,f,a,al) != bl ) return(-1);
	else return(0);
}

/* int spc_read_socket()
 * Receive a message from a socket
 *
 * Argument : socket descriptor
 *            message pt
 *	      message length
 *            flags
 *            destination pt
 *	      destination length
 *
 * Return   : 0 if OK / -1 if error
 */
int spc_read_socket(WDX_SOCKET s,char *b,int bl, int f,struct sockaddr *a,int *al)
{
int _ret;

   _ret = recvfrom(s,b,bl,f,a,al);
   if ( _ret != bl )
		return(-1);
   else return(0);
}

/* int spc_select_socket()
 * Select on a read socket descriptor
 *
 * Argument : socket descriptor
 *            timeval pt
 *
 * Return   : 0 if OK / -1 if error
 */
int spc_select_socket(WDX_SOCKET s,struct timeval *timeout)
{
int readfds;            /* variable needed for select(2) */
int writefds;           /* variable needed for select(2) */
int exceptfds;          /* variable needed for select(2) */
int nfds;               /* variable needed for select(2) */

#ifdef DEBUG
		nfds = 1;
		readfds = 0;
#else
		nfds = s + 1;
		readfds = 1 << s;
#endif
		writefds = 0;
		exceptfds = 0;
	return( select(nfds,(void*)&readfds,(void*)&writefds,(void*)&exceptfds,timeout));

}

/* int spc_get_ifconfig()
 * for a device name get :
 *      interface name
 *      netmask address
 *      broadcast address
 * Argument : interface name pt
 *	      struct in_addr pt for interface address
 *	      struct in_addr pt for netmask address
 *	      struct in_addr pt for broadcast address
 *
 * return : 0 ok, -1 error
 */
int spc_get_ifconfig (char *ifname,struct in_addr *ifaddr,struct in_addr *netmask,struct in_addr *brdaddr)
{
 struct ifreq         _ifr;	/* for use by ioctl */
 struct sockaddr_in _saddr;	/* for byte copy operation */
 WDX_SOCKET             _d;	/* descriptor */

 /* check the number of chars in interface */
 strncpy(_ifr.ifr_name, ifname, sizeof(_ifr.ifr_name));

 /* open a socket for use by ioctl later on */
 if ( spc_open_socket (&_d,AF_INET, SOCK_DGRAM, 0) == -1 )
 {
	  PERROR_B
	  "Can't create a temporary endpoint for communication %m"
	  PERROR_E;
	  return ( -1 );
 }

 /* get the interface address  */
 if ( ioctl(_d, SIOCGIFADDR, (char *)&_ifr) == -1 )
 {
	  sprintf(messagestring,
			  "Can't get ifaddr for interface %s %%m",ifname);
	  PERROR_B
	  messagestring
	  PERROR_E;
	  spc_close_socket(_d);
	  return ( -1 );
 }
 else
 {
	  memcpy(&_saddr, &_ifr.ifr_addr, sizeof(_ifr.ifr_addr));
	  *ifaddr = _saddr.sin_addr;
 }

 /* get the netmask */
 if ( ioctl(_d, SIOCGIFNETMASK, (char *)&_ifr) == -1 )
 {
	  sprintf(messagestring,
			  "Can't get netmask address for interface %s %%m",ifname);
	  PERROR_B
	  messagestring
	  PERROR_E;
	  spc_close_socket(_d);
	  return ( -1 );
 }
 else
 {
	  memcpy(&_saddr, &_ifr.ifr_addr, sizeof(_ifr.ifr_addr));
	  *netmask = _saddr.sin_addr;
 }

 /* get the broadcast addr */
 if ( ioctl(_d, SIOCGIFBRDADDR, (char *)&_ifr) == -1 )
 {
	  sprintf(messagestring,
			  "Can't get broadcast address for interface %s %%m",ifname);
	  PERROR_B
	  messagestring
	  PERROR_E;
	  spc_close_socket(_d);
	  return ( -1 );
 }
 else
 {
	  memcpy(&_saddr, &_ifr.ifr_broadaddr, sizeof(_ifr.ifr_broadaddr));
	  *brdaddr = _saddr.sin_addr;
 }

 spc_close_socket(_d);
 return (0);

}

/* int spc_open_fifo()
 * Open a FIFO according to specified mode
 * The wdxdirectory path is added to the fifo name to create a complete file
 *
 * Argument : pt to FIFO descriptor
 *            pt to FIFO name
 *	      message size
 *	      number of message in fifo
 *            mode
 *
 * Return   : 0 if OK / -1 if error
 */
int spc_open_fifo(WDX_FIFO *f,char *fifoname,int size,
											int nbmess,enum wdx_modes mode)
{
char name[FIFO_LEN];

	sprintf(name,"%s%s",wdxdirectory,fifoname);
	*f = -1;

	if ( mode == WDX_APPLICATION )
	{

/* modified DM 16/10/03 */
#ifdef OpenBSD
		if (mkfifo (name, S_IWGRP|S_IWOTH) == -1 )
			return(-1);
#else
		if ( mknod(name,S_IFIFO|PERM,0) == -1 )
			return(-1);
#endif

		*f = open(name,O_RDONLY|O_NDELAY,PERM);
	}

	else /* WDX_PROCESS : write only */

	{
		*f = open(name,O_WRONLY|O_NDELAY,PERM);
		signal(SIGPIPE,SIG_IGN);
	}

	if ( *f == -1 )
		return(-1);

	set_fifo_blocking(*f); /* FIFO set to blocking mode */
	return(0);
}

/* int spc_close_fifo()
 * Close a FIFO and if mode is WDX_APPLICATION delete the FIFO
 * The wdxdirectory path is added to the fifo name to create a complete file
 *
 * Argument : FIFO decriptor
 *            pt to FIFO name
 *	      mode
 *
 * Return   : 0 if OK / -1 if error
 */
int spc_close_fifo(WDX_FIFO f,char *fifoname,enum wdx_modes mode)
{
char name[FIFO_LEN];

   sprintf(name,"%s%s",wdxdirectory,fifoname);
   if ( f != -1 )
	   close(f);

   if ( mode == WDX_APPLICATION )
	   unlink(name);

   return(0);
}

/* void spc_cancel_fifo()
 * Cancel pending IO
 * Nothing is done for ULTRIX
 *
 * Argument : FIFO decriptor
 *
 * Return   : None
 */
void spc_cancel_fifo(WDX_FIFO f)
{
int i;
   i = 0;
}

/* int spc_write_fifo()
 * Write to a FIFO
 *
 * Argument : FIFO descriptor
 *	      pt to buffer
 *	      buffer length
 *
 * Return   : 0 if OK / -1 if error / -2 if FIFO full
 */
int spc_write_fifo(WDX_FIFO f, char *buffer, int length)
{
int _ret;

	_ret = 0;
	signal(SIGPIPE,SIG_IGN);
	set_fifo_nonblocking(f);

	if ( write(f,buffer,length) != length )
	{
		if ( errno == EWOULDBLOCK )
			_ret = -2;
		else
			_ret = -1;
	}
	return(_ret);
}

/* int spc_read_fifo()
 * Read a FIFO
 *
 * Argument : FIFO descriptor
 *	      pt to buffer
 *	      buffer length
 *
 * Return   : 0 if OK / -1 if error
 */
int spc_read_fifo(WDX_FIFO f, char *buffer, int length)
{
int r;

/*  set_fifo_blocking(f); */
	r = read (f,buffer,length);

	if (r >= 0 )
		return(r);
	else
		return(-1);
}

/* int spc_process_running()
 * Check if the wdx process is running
 *
 * Argument : pid
 *
 * Return   : 0 : running / -1 not running
 */
int spc_process_running(pid_t pid)
{

   if ( kill(pid,0) == 0 )
		return(0);
   else return(-1);
}

/* int spc_super_user()
 * Check if the process has super user priviledges
 *
 * Argument : none
 *
 * Return : 0 not super user / 1 super user
 */
int spc_super_user()
{
  if ( geteuid() )
	   return(0);
  else return(1);
}

void spc_unlock(int lockid)
{
}
int spc_set_lock(char *s)
{
	return(0);
}
int spc_lookat_daemon_lock(char *s,void (*funct)(),int slot)
{
	return(0);
}
int spc_lookat_app_lock(char *s,void (*funct)(),pid_t pid )
{
	return(0);
}

/* struct tm *spc_localtime()
 */
struct tm *spc_localtime(time_t *t)
{
 return(localtime(t));
}

/* int spc_build_generation()
 */
time_t spc_build_generation()
{

/* DM 14/09/03 replace 'time()' with 'gettimeofday()' */

/*
time_t tt;

	tt = time((long *)0);
	return(tt);
*/

struct timeval timeofday;	/* used to get internal clock value */
struct timezone tz;

	gettimeofday (&timeofday, &tz); /* used to initialize value for activity algorithm */
	return ((time_t)timeofday.tv_sec);
}

/* spc_communication_init()
 * Empty routine
 *
 * Argument : None
 *
 * Return   : None
 */
void spc_communication_init()
{
}

/*
 * LOCAL ROUTINES
 */

/* int set_fifo_nonblocking()
 * Set a fifo to nonblocking mode.
 *
 * Argument :  descriptor
 *
 * Return   :  o if OK
 *            -1 if error
 */
int set_fifo_nonblocking(d)
int d;
{
int _s;

   _s = fcntl(d,F_SETFL,O_NDELAY | fcntl(d,F_GETFL,0));
   return(_s);
}

/* int set_fifo_blocking()
 * Set a fifo to blocking mode.
 *
 * Argument :  descriptor
 *
 * Return   :  o if OK
 *            -1 if error
 */
int set_fifo_blocking(d)
int d;
{
int _s;

   _s = fcntl(d,F_SETFL,~O_NDELAY & fcntl(d,F_GETFL,0));
   return(_s);
}


int spc_help(FILE *logdes)
{
	return(0);
}

void spc_sleep()
{
   signal(SIGALRM, sleep_timer_exp);
   ualarm(400000, 0);
   sigblock(sigmask(SIGALRM));
   while (sleep_finished_flag == 0)
	   sigpause(0);
   sleep_finished_flag = 0;
}

/* MK 0001 DEBUG ROUTINE TO OUTPUT TIMESTAMP */
/* DM 14/09/03 Commented out as not used now */
/*
 void spc_output_time (char *message_str)
{
int status;
struct timeval tp1;
struct tm *tp_ptr;

status = gettimeofday(&tp1, &tz);

if (status == 0)
{

   tp_ptr = localtime(&(tp1.tv_sec));
   fprintf(stderr, "\n\n%-35s : %4d-%02d-%02d %02d:%02d:%02d.%09ld\n",
		  message_str,
		  tp_ptr->tm_year, tp_ptr->tm_mon+1, tp_ptr->tm_mday,
		  tp_ptr->tm_hour, tp_ptr->tm_min, tp_ptr->tm_sec, tp1.tv_usec);
}
else
   fprintf(stderr, "\ngetclock error occurred");
}
*/
