/*
 *   WDX: WitchDoctor-WDX, versatile distributed lock manager
 *   Copyright (C) 2003 Dominique Chabord,
 *   Please refer to credits section below
 *
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   ************************************************
 *
 *   History and credits:
 *   First version of WitchDoctor-WDX was based on:
 *
 *   DECwatchdog/Autopilot, watchdog and lock manager
 *   Copyright (C) 2003 HP
 *
 *   Developed by
 *     Computer Special Systems, Annecy France
 *     A Division of Digital Equipment Corporation
 *   and
 *     Custom-Systems, Annecy France
 *     An engineering center of Compaq
 *
 *   Provided under GNU-GPL terms by HP, 30 June 2003
 *   GNU-GPL replaces previous license statements
 *
 *   Rebranded, ported and modified for Shaman-X project, 15 July 2003
 *   by Dominique Chabord
 *   and maintained by other contributors as mentionned in the code
 *
 *   ************************************************
 *
 *   WDX is promoted by Shaman-X project
 *
 *   Check for new versions and documentation at
 *   http://www.shaman-x.org
 *
 *   ************************************************
 *
 *   Most of WDX code is designed to run unchanged on
 *   different operating systems, including at V1.0 time:
 *   Gnu-Linux, WindowsNT, DigitalUNIX and OpenVMS.
 *   Operating system specifics are identified by dedicated files.
 *
 *   Evolutions
 *   ----------
 *   July-2003     Dominique Chabord    WitchDoctor-WDX V1.0
 *                 Creation
 *
 *   ************************************************
 *
 *
 *	wdx_ui.h
 *
 *	User interface definition
 *
 ************************************************************************/
/*
 *
 *  05-Mar-94 -- Cornu
 *      Creation
 */

/*
 * USER INTERFACE definition
 */

#include "wdx_api.h"

#define SINGLEOPTION    1

#define WDX_SEPARATOR '-'

#define WDX_FATAL_ERROR -1
#define WDX_RETURN_SUCCESS 0
#define WDX_RETURN_ERROR 1
#define WDX_RETURN_SATELLITE 2

enum wdx_no_option { WDX_NO_OPTION };

struct userinterface
{
   char *option;                /* option keyword */
   int  (*controlfunct)();      /* associated control function */
   int  (*actionfunct)();	/* associated action function */
   char *usage;                 /* usage */
   enum wdx_type    type;       /* command type */
   int           command;	/* command value */
};

/* Control declaration */
int     ui_daemoncontrol(char *p);
int     ui_noargcontrol(char *p);
int     ui_errorcontrol(char *p);
int     ui_resourcecontrol(char *p);
int     ui_resourcecontrol2(char *p);
int     ui_supercontrol(char *p);
int     ui_pausecontrol(char *p);
int     ui_snapcontrol(char *p);

/* Action declaration */
int ui_daemon(enum wdx_type type,int command,FILE *logdes);
int ui_application(enum wdx_type type,int command,FILE *logdes);
int ui_constant(enum wdx_type type,int command,FILE *logdes);
int ui_show(enum wdx_type type, int command,FILE *logdes);
int ui_stop(enum wdx_type type, int command,FILE *logdes);

struct userinterface ui[]=
{
        { "-daemon",
          ui_daemoncontrol,
	  ui_daemon,
          "Start local process :                        %s [mission] -daemon [membername]\n" ,
          WDX_NO_TYPE,
          WDX_NO_OPTION },
        { "-join",
          ui_supercontrol,
	  ui_application,
          "Join the mission :                           %s [mission] -join\n" ,
          WDX_COMMAND,
          WDX_MEMBER_UP },
        { "-leave",
          ui_supercontrol,
	  ui_application,
          "Kill local process :                         %s [mission] -leave\n" ,
          WDX_COMMAND,
          WDX_PROCESS_KILL},
        { "-run",
          ui_supercontrol,
	  ui_application,
          "Initialise mission :                         %s [mission] -run\n" ,
          WDX_COMMAND,
          WDX_GROUP_RUN },
        { "-kill",
          ui_supercontrol,
	  ui_application,
          "Kill all processes of the mission :          %s [mission] -kill\n" ,
          WDX_COMMAND,
          WDX_GROUP_KILL },
        { "-exception",
          ui_supercontrol,
	  ui_application,
          "Start exceptionscript on all members :       %s [mission] -exception\n" ,
          WDX_COMMAND,
          WDX_GROUP_EXCEPTION },
        { "-stop",
          ui_snapcontrol,
	  ui_stop,
          "Suspend mission :                            %s [mission] -stop [snapshot]\n" ,
          WDX_COMMAND,
          WDX_GROUP_STOP },
        { "-start",
          ui_snapcontrol,
	  ui_application,
          "Restore a context and start mission :        %s [mission] -start [snapshot]\n" ,
          WDX_COMMAND,
          WDX_GROUP_START },
        { "-pause",
          ui_pausecontrol,
	  ui_stop,
          "Suspend mission for a while :                %s [mission] -pause number_of_second\n" ,
          WDX_COMMAND,
          WDX_GROUP_PAUSE },
        { "-master",
          ui_supercontrol,
	  ui_application,
          "Try and lock mastertoken :                   %s [mission] -master\n" ,
          WDX_REQUEST,
          WDX_MASTER },
        { "-getmaster",
          ui_supercontrol,
	  ui_application,
          "Force a lock on mastertoken :                %s [mission] -getmaster\n" ,
          WDX_REQUEST,
          WDX_GETMASTER },
        { "-nomaster",
          ui_supercontrol,
	  ui_application,
          "Release the lock on mastertoken :            %s [mission] -nomaster\n" ,
          WDX_REQUEST,
          WDX_NOMASTER},
        { "-declare",
          ui_resourcecontrol,
	  ui_application,
          "Declare a resource|directory :               %s [mission] -declare resource|directory\n" ,
          WDX_REQUEST,
          WDX_DECLARE},
        { "-erase",
          ui_resourcecontrol,
	  ui_application,
          "Erase a resource|directory :                 %s [mission] -erase resource|directory\n" ,
          WDX_REQUEST,
          WDX_ERASE},
        { "-reserve",
          ui_resourcecontrol,
	  ui_application,
          "Try and lock a resource :                    %s [mission] -reserve resource\n" ,
          WDX_REQUEST,
          WDX_RESERVE},
        { "-getresource",
          ui_resourcecontrol,
	  ui_application,
          "Force a lock on a resource :                 %s [mission] -getresource resource\n" ,
          WDX_REQUEST,
          WDX_GET},
        { "-mount",
          ui_resourcecontrol,
	  ui_application,
          "Try and lock a mounted directory :           %s [mission] -mount directory\n" ,
          WDX_REQUEST,
          WDX_MOUNT},
        { "-free",
          ui_resourcecontrol,
	  ui_application,
          "Release the lock on a resource|directory :   %s [mission] -free resource|directory\n" ,
          WDX_REQUEST,
          WDX_FREE},
        { "-fault",
          ui_resourcecontrol,
	  ui_application,
          "Announce a resource|directory fault state :  %s [mission] -fault resource|directory\n" ,
          WDX_REQUEST,
          WDX_FAULT},
        { "-waitreserve",
          ui_resourcecontrol2,
	  ui_application,
          "Wait for resource until reserved :           %s [mission] -waitreserve resource\n" ,
          WDX_REQUEST,
          WDX_WAIT_FOR_RESERVE},
        { "-waitmount",
          ui_resourcecontrol2,
	  ui_application,
          "Wait for a directory until mounted :         %s [mission] -waitmount directory\n" ,
          WDX_REQUEST,
          WDX_WAIT_FOR_MOUNT},
        { "-list",
          ui_noargcontrol,
	  ui_application,
          "List mission status :                        %s [mission] -list\n" ,
          WDX_LOCAL,
          WDX_LIST},
        { "-version",
          ui_noargcontrol,
	  ui_constant,
          "List configuration values :                  %s [mission] -version\n" ,
          WDX_LOCAL,
          WDX_CONSTANT},
        { "-namemaster",
          ui_noargcontrol,
	  ui_application,
          "Print name of masterholder :                 %s [mission] -namemaster\n" ,
          WDX_LOCAL,
          WDX_MASTERNAME},
        { "-status",
          ui_noargcontrol,
	  ui_application,
          "Print status of local member :               %s [mission] -status\n" ,
          WDX_LOCAL,
          WDX_STATUS},
        { "-log",
          ui_supercontrol,
	  ui_application,
          "Rename log file :                            %s [mission] -log\n" ,
          WDX_COMMAND,
          WDX_LOG},
        { "-show",
          ui_noargcontrol,
	  ui_show,
          "Print local processing missions :            %s -show\n" ,
	  WDX_LOCAL,
          WDX_NO_OPTION},
        { 0,
          ui_errorcontrol,
	  ui_errorcontrol,
          "Not an option" ,
          WDX_NO_TYPE,
          WDX_NO_OPTION }
};

